cdef class Group:

    """
    Group of processes
    """

    def __cinit__(self, Group group: Optional[Group] = None):
        self.ob_mpi = MPI_GROUP_NULL
        if group is None: return
        self.ob_mpi = group.ob_mpi

    def __dealloc__(self):
        if not (self.flags & PyMPI_OWNED): return
        CHKERR( del_Group(&self.ob_mpi) )

    def __richcmp__(self, other, int op):
        if not isinstance(other, Group): return NotImplemented
        cdef Group s = <Group>self, o = <Group>other
        if   op == Py_EQ: return (s.ob_mpi == o.ob_mpi)
        elif op == Py_NE: return (s.ob_mpi != o.ob_mpi)
        cdef mod = type(self).__module__
        cdef cls = type(self).__name__
        raise TypeError("unorderable type: '%s.%s'" % (mod, cls))

    def __bool__(self) -> bool:
        return self.ob_mpi != MPI_GROUP_NULL

    # Group Accessors
    # ---------------

    def Get_size(self) -> int:
        """
        Return the size of a group
        """
        cdef int size = -1
        CHKERR( MPI_Group_size(self.ob_mpi, &size) )
        return size

    property size:
        """number of processes in group"""
        def __get__(self) -> int:
            return self.Get_size()

    def Get_rank(self) -> int:
        """
        Return the rank of this process in a group
        """
        cdef int rank = -1
        CHKERR( MPI_Group_rank(self.ob_mpi, &rank) )
        return rank

    property rank:
        """rank of this process in group"""
        def __get__(self) -> int:
            return self.Get_rank()

    @classmethod
    def Translate_ranks(
        cls,
        Group group1: Group,
        ranks1: Sequence[int],
        Group group2: Optional[Group] = None,
    ) -> List[int]:
        """
        Translate the ranks of processes in
        one group to those in another group
        """
        cdef MPI_Group grp1 = MPI_GROUP_NULL
        cdef MPI_Group grp2 = MPI_GROUP_NULL
        cdef int i = 0, n = 0, *iranks1 = NULL, *iranks2 = NULL
        cdef tmp1 = getarray(ranks1, &n, &iranks1)
        cdef tmp2 = newarray(n, &iranks2)
        #
        grp1 = group1.ob_mpi
        if group2 is not None:
            grp2 = group2.ob_mpi
        else:
            CHKERR( MPI_Comm_group(MPI_COMM_WORLD, &grp2) )
        try:
            CHKERR( MPI_Group_translate_ranks(grp1, n, iranks1,
                                              grp2, iranks2) )
        finally:
            if group2 is None:
                CHKERR( MPI_Group_free(&grp2) )
        #
        cdef object ranks2 = [iranks2[i] for i from 0 <= i < n]
        return ranks2

    @classmethod
    def Compare(cls, Group group1: Group, Group group2: Group) -> int:
        """
        Compare two groups
        """
        cdef int flag = MPI_UNEQUAL
        CHKERR( MPI_Group_compare(group1.ob_mpi, group2.ob_mpi, &flag) )
        return flag

    # Group Constructors
    # ------------------

    def Dup(self) -> Group:
        """
        Duplicate a group
        """
        cdef Group group = Group.__new__(Group)
        CHKERR( MPI_Group_union(self.ob_mpi, MPI_GROUP_EMPTY, &group.ob_mpi) )
        return group

    @classmethod
    def Union(cls, Group group1: Group, Group group2: Group) -> Group:
        """
        Produce a group by combining
        two existing groups
        """
        cdef Group group = Group.__new__(Group)
        CHKERR( MPI_Group_union(
                group1.ob_mpi, group2.ob_mpi, &group.ob_mpi) )
        return group

    @classmethod
    def Intersection(cls, Group group1: Group, Group group2: Group) -> Group:
        """
        Produce a group as the intersection
        of two existing groups
        """
        cdef Group group = Group.__new__(Group)
        CHKERR( MPI_Group_intersection(
                group1.ob_mpi, group2.ob_mpi, &group.ob_mpi) )
        return group

    Intersect = Intersection

    @classmethod
    def Difference(cls, Group group1: Group, Group group2: Group) -> Group:
        """
        Produce a group from the difference
        of two existing groups
        """
        cdef Group group = Group.__new__(Group)
        CHKERR( MPI_Group_difference(
                group1.ob_mpi, group2.ob_mpi, &group.ob_mpi) )
        return group

    def Incl(self, ranks: Sequence[int]) -> Group:
        """
        Produce a group by reordering an existing
        group and taking only listed members
        """
        cdef int n = 0, *iranks = NULL
        ranks = getarray(ranks, &n, &iranks)
        cdef Group group = Group.__new__(Group)
        CHKERR( MPI_Group_incl(self.ob_mpi, n, iranks, &group.ob_mpi) )
        return group

    def Excl(self, ranks: Sequence[int]) -> Group:
        """
        Produce a group by reordering an existing
        group and taking only unlisted members
        """
        cdef int n = 0, *iranks = NULL
        ranks = getarray(ranks, &n, &iranks)
        cdef Group group = Group.__new__(Group)
        CHKERR( MPI_Group_excl(self.ob_mpi, n, iranks, &group.ob_mpi) )
        return group

    def Range_incl(self, ranks: Sequence[Tuple[int, int, int]]) -> Group:
        """
        Create a new group from ranges of
        of ranks in an existing group
        """
        cdef int *p = NULL, (*ranges)[3]# = NULL ## XXX cython fails
        ranges = NULL
        cdef int i = 0, n = <int>len(ranks)
        cdef tmp1 = allocate(n, sizeof(int[3]), &ranges)
        for i from 0 <= i < n:
            p = <int*> ranges[i]
            p[0], p[1], p[2] = ranks[i]
        cdef Group group = Group.__new__(Group)
        CHKERR( MPI_Group_range_incl(self.ob_mpi, n, ranges, &group.ob_mpi) )
        return group

    def Range_excl(self, ranks: Sequence[Tuple[int, int, int]]) -> Group:
        """
        Create a new group by excluding ranges
        of processes from an existing group
        """
        cdef int *p = NULL, (*ranges)[3]# = NULL ## XXX cython fails
        ranges = NULL
        cdef int i = 0, n = <int>len(ranks)
        cdef tmp1 = allocate(n, sizeof(int[3]), &ranges)
        for i from 0 <= i < n:
            p = <int*> ranges[i]
            p[0], p[1], p[2] = ranks[i]
        cdef Group group = Group.__new__(Group)
        CHKERR( MPI_Group_range_excl(self.ob_mpi, n, ranges, &group.ob_mpi) )
        return group

    # Group Destructor
    # ----------------

    def Free(self) -> None:
        """
        Free a group
        """
        CHKERR( MPI_Group_free(&self.ob_mpi) )
        if self is __GROUP_EMPTY__: self.ob_mpi = MPI_GROUP_EMPTY

    # Fortran Handle
    # --------------

    def py2f(self) -> int:
        """
        """
        return MPI_Group_c2f(self.ob_mpi)

    @classmethod
    def f2py(cls, arg: int) -> Group:
        """
        """
        cdef Group group = Group.__new__(Group)
        group.ob_mpi = MPI_Group_f2c(arg)
        return group



cdef Group __GROUP_NULL__  = new_Group ( MPI_GROUP_NULL  )
cdef Group __GROUP_EMPTY__ = new_Group ( MPI_GROUP_EMPTY )


# Predefined group handles
# ------------------------

GROUP_NULL  = __GROUP_NULL__   #: Null group handle
GROUP_EMPTY = __GROUP_EMPTY__  #: Empty group handle



import sys
from mpi4py.futures import MPIPoolExecutor


def main():
    def getarg(opt, default=None):
        try:
            return sys.argv[sys.argv.index('--'+opt)+1]
        except ValueError:
            return default

    options = {}
    if '--host' in sys.argv or '--port' in sys.argv:
        service = (getarg('host'), getarg('port'))
    else:
        service = getarg('service')
    if '--info' in sys.argv:
        info = getarg('info').split(',')
        info = dict(entry.split('=') for entry in info if entry)
    else:
        info = None

    with MPIPoolExecutor(service=service, mpi_info=info) as executor:
        fut1 = executor.submit(abs, +42)
        fut2 = executor.submit(abs, -42)
    assert fut1.result(0) == 42
    assert fut2.result(0) == 42


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# If you want VampirTrace to log MPI calls, you have to add the two
# lines below at the very beginning of your main bootstrap script.
import mpi4py
mpi4py.rc.threads = False
mpi4py.profile('vt-mpi', logfile='ring')

from mpi4py import MPI

comm = MPI.COMM_WORLD
size = comm.Get_size()
rank = comm.Get_rank()

src  = rank-1
dest = rank+1
if rank == 0:
    src = size-1
if rank == size-1:
    dest = 0

try:
    from numpy import zeros
    a1 = zeros(1000000, 'd')
    a2 = zeros(1000000, 'd')
except ImportError:
    from array import array
    a1 = array('d', [0]*1000); a1 *= 1000
    a2 = array('d', [0]*1000); a2 *= 1000

comm.Sendrecv(sendbuf=a1, recvbuf=a2,
              source=src, dest=dest)

MPI.Request.Waitall([
    comm.Isend(a1, dest=dest),
    comm.Irecv(a2, source=src),
    ])



from __future__ import print_function
from __future__ import division

try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen

from mpi4py.futures import MPIPoolExecutor

URLS = [
    'http://www.google.com/',
    'http://www.apple.com/',
    'http://www.ibm.com/',
    'http://www.slashdot.org/',
    'http://www.python.org/',
    'http://www.bing.com/',
    'http://www.facebook.com/',
    'http://www.yahoo.com/',
    'http://www.youtube.com/',
    'http://www.blogger.com/',
]

def load_url(url):
    return url, urlopen(url).read()

def test_crawl():
    with MPIPoolExecutor(10) as executor:
        for url, content in executor.map(load_url, URLS,
                                         timeout=10, unordered=True):
            print('%-25s: %6.2f KiB' % (url, len(content)/(1 << 10)))

if __name__ == '__main__':
    test_crawl()



# -----------------------------------------------------------------------------

def _typecode(Datatype datatype: Datatype) -> Optional[str]:
    """
    Map MPI datatype to typecode string
    """
    cdef const char *tc = Datatype2String(datatype.ob_mpi)
    return pystr(tc) if tc != NULL else None

# -----------------------------------------------------------------------------

cdef inline const char* Datatype2String(MPI_Datatype datatype) nogil:
    if datatype == MPI_DATATYPE_NULL: return NULL
    # MPI
    elif datatype == MPI_LB     : return NULL
    elif datatype == MPI_UB     : return NULL
    elif datatype == MPI_PACKED : return "B"
    elif datatype == MPI_BYTE   : return "B"
    elif datatype == MPI_AINT   : return "p"
    elif datatype == MPI_OFFSET :
        if   sizeof(MPI_Offset) == sizeof(MPI_Aint)  : return "p"
        elif sizeof(MPI_Offset) == sizeof(long long) : return "q"
        elif sizeof(MPI_Offset) == sizeof(long)      : return "l"
        elif sizeof(MPI_Offset) == sizeof(int)       : return "i"
        else                                         : return NULL
    elif datatype == MPI_COUNT :
        if   sizeof(MPI_Count)  == sizeof(MPI_Aint)  : return "p"
        elif sizeof(MPI_Count)  == sizeof(long long) : return "q"
        elif sizeof(MPI_Count)  == sizeof(long)      : return "l"
        elif sizeof(MPI_Count)  == sizeof(int)       : return "i"
        else                                         : return NULL
    # C - character
    elif datatype == MPI_CHAR  : return "c"
    elif datatype == MPI_WCHAR :
        if sizeof(wchar_t) == 4: return "U"
        else                   : return NULL
    # C - (signed) integral
    elif datatype == MPI_SIGNED_CHAR : return "b"
    elif datatype == MPI_SHORT       : return "h"
    elif datatype == MPI_INT         : return "i"
    elif datatype == MPI_LONG        : return "l"
    elif datatype == MPI_LONG_LONG   : return "q"
    # C - unsigned integral
    elif datatype == MPI_UNSIGNED_CHAR      : return "B"
    elif datatype == MPI_UNSIGNED_SHORT     : return "H"
    elif datatype == MPI_UNSIGNED           : return "I"
    elif datatype == MPI_UNSIGNED_LONG      : return "L"
    elif datatype == MPI_UNSIGNED_LONG_LONG : return "Q"
    # C - (real) floating
    elif datatype == MPI_FLOAT       : return "f"
    elif datatype == MPI_DOUBLE      : return "d"
    elif datatype == MPI_LONG_DOUBLE : return "g"
    # C99 - boolean
    elif datatype == MPI_C_BOOL : return "?"
    # C99 - integral
    elif datatype == MPI_INT8_T   : return "i1"
    elif datatype == MPI_INT16_T  : return "i2"
    elif datatype == MPI_INT32_T  : return "i4"
    elif datatype == MPI_INT64_T  : return "i8"
    elif datatype == MPI_UINT8_T  : return "u1"
    elif datatype == MPI_UINT16_T : return "u2"
    elif datatype == MPI_UINT32_T : return "u4"
    elif datatype == MPI_UINT64_T : return "u8"
    # C99 - complex floating
    elif datatype == MPI_C_COMPLEX             : return "F"
    elif datatype == MPI_C_FLOAT_COMPLEX       : return "F"
    elif datatype == MPI_C_DOUBLE_COMPLEX      : return "D"
    elif datatype == MPI_C_LONG_DOUBLE_COMPLEX : return "G"
    # C++ - boolean
    elif datatype == MPI_CXX_BOOL : return "?"
    # C++ - complex floating
    elif datatype == MPI_CXX_FLOAT_COMPLEX       : return "F"
    elif datatype == MPI_CXX_DOUBLE_COMPLEX      : return "D"
    elif datatype == MPI_CXX_LONG_DOUBLE_COMPLEX : return "G"
    # Fortran
    elif datatype == MPI_CHARACTER        : return "c"
    elif datatype == MPI_LOGICAL          : return NULL
    elif datatype == MPI_INTEGER          : return "i"
    elif datatype == MPI_REAL             : return "f"
    elif datatype == MPI_DOUBLE_PRECISION : return "d"
    elif datatype == MPI_COMPLEX          : return "F"
    elif datatype == MPI_DOUBLE_COMPLEX   : return "D"
    # Fortran 90
    elif datatype == MPI_LOGICAL1  : return NULL
    elif datatype == MPI_LOGICAL2  : return NULL
    elif datatype == MPI_LOGICAL4  : return NULL
    elif datatype == MPI_LOGICAL8  : return NULL
    elif datatype == MPI_INTEGER1  : return "i1"
    elif datatype == MPI_INTEGER2  : return "i2"
    elif datatype == MPI_INTEGER4  : return "i4"
    elif datatype == MPI_INTEGER8  : return "i8"
    elif datatype == MPI_INTEGER16 : return "i16"
    elif datatype == MPI_REAL2     : return "f2"
    elif datatype == MPI_REAL4     : return "f4"
    elif datatype == MPI_REAL8     : return "f8"
    elif datatype == MPI_REAL16    : return "f16"
    elif datatype == MPI_COMPLEX4  : return "c4"
    elif datatype == MPI_COMPLEX8  : return "c8"
    elif datatype == MPI_COMPLEX16 : return "c16"
    elif datatype == MPI_COMPLEX32 : return "c32"

    else : return NULL

# -----------------------------------------------------------------------------



"""
Compare the speed of downloading URLs sequentially vs. using futures.
"""

import sys
import time
import functools

try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen

try:
    from concurrent.futures import ThreadPoolExecutor
except ImportError:
    ThreadPoolExecutor = lambda n: None
try:
    from concurrent.futures import ProcessPoolExecutor
except ImportError:
    ProcessPoolExecutor = lambda n: None

from mpi4py.futures import MPIPoolExecutor, as_completed

URLS = [
    'http://www.google.com/',
    'http://www.apple.com/',
    'http://www.ibm.com',
    'http://www.thisurlprobablydoesnotexist.com',
    'http://www.slashdot.org/',
    'http://www.python.org/',
    'http://www.bing.com/',
    'http://www.facebook.com/',
    'http://www.yahoo.com/',
    'http://www.youtube.com/',
    'http://www.blogger.com/',
]

def load_url(url, timeout):
    kwargs = {'timeout': timeout} if sys.version_info >= (2, 6) else {}
    return urlopen(url, **kwargs).read()

def download_urls_sequential(urls, timeout=60):
    url_to_content = {}
    for url in urls:
        try:
            url_to_content[url] = load_url(url, timeout=timeout)
        except:
            pass
    return url_to_content

def download_urls_with_executor(executor, urls, timeout=60):
    if executor is None: return {}
    try:
        url_to_content = {}
        future_to_url = dict((executor.submit(load_url, url, timeout), url)
                             for url in urls)
        for future in as_completed(future_to_url):
            try:
                url_to_content[future_to_url[future]] = future.result()
            except:
                pass
        return url_to_content
    finally:
        executor.shutdown()

def main():
    for meth, fn in [('sequential',
                      functools.partial(download_urls_sequential,
                                        URLS)),
                     ('threads',
                      functools.partial(download_urls_with_executor,
                                        ThreadPoolExecutor(10), URLS)),
                     ('processes',
                      functools.partial(download_urls_with_executor,
                                        ProcessPoolExecutor(10), URLS)),
                     ('mpi4py',
                      functools.partial(download_urls_with_executor,
                                        MPIPoolExecutor(10), URLS))]:
        sys.stdout.write('%s: ' % meth.ljust(11))
        sys.stdout.flush()
        start = time.time()
        url_map = fn()
        elapsed = time.time() - start
        sys.stdout.write('%5.2f seconds (%2d of %d downloaded)\n' %
                         (elapsed, len(url_map), len(URLS)))
        sys.stdout.flush()

if __name__ == '__main__':
    main()



from mpi4py import MPI
import mpiunittest as unittest
import arrayimpl
import sys

pypy_lt_53 = (hasattr(sys, 'pypy_version_info') and
              sys.pypy_version_info < (5, 3))

def mkzeros(n):
    if pypy_lt_53:
        return b'\0' * n
    return bytearray(n)

def memzero(m):
    try:
        m[:] = 0
    except IndexError: # cffi buffer
        m[0:len(m)] = b'\0'*len(m)

class BaseTestRMA(object):

    COMM = MPI.COMM_NULL
    INFO = MPI.INFO_NULL

    COUNT_MIN = 0

    def setUp(self):
        nbytes = 100*MPI.DOUBLE.size
        try:
            self.mpi_memory = MPI.Alloc_mem(nbytes)
            self.memory = self.mpi_memory
            memzero(self.memory)
        except MPI.Exception:
            import array
            self.mpi_memory = None
            self.memory = array.array('B',[0]*nbytes)
        self.WIN = MPI.Win.Create(self.memory, 1, self.INFO, self.COMM)

    def tearDown(self):
        self.WIN.Free()
        if self.mpi_memory:
            MPI.Free_mem(self.mpi_memory)

    def testPutGet(self):
        group = self.WIN.Get_group()
        size = group.Get_size()
        group.Free()
        for array, typecode in arrayimpl.subTest(self):
            if unittest.is_mpi_gpu('mvapich2', array): continue
            for count in range(self.COUNT_MIN, 10):
                for rank in range(size):
                    sbuf = array([rank]*count, typecode)
                    rbuf = array(-1, typecode, count+1)
                    self.WIN.Fence()
                    self.WIN.Lock(rank)
                    r = self.WIN.Rput(sbuf.as_mpi(), rank)
                    r.Wait()
                    self.WIN.Flush(rank)
                    r = self.WIN.Rget(rbuf.as_mpi_c(count), rank)
                    r.Wait()
                    self.WIN.Unlock(rank)
                    for i in range(count):
                        self.assertEqual(sbuf[i], rank)
                        self.assertEqual(rbuf[i], rank)
                    self.assertEqual(rbuf[-1], -1)

    @unittest.skipMPI('openmpi(>=1.10.0,<1.11.0)')
    def testAccumulate(self):
        group = self.WIN.Get_group()
        size = group.Get_size()
        group.Free()
        for array, typecode in arrayimpl.subTest(self):
            if unittest.is_mpi_gpu('openmpi', array): continue
            if unittest.is_mpi_gpu('mvapich2', array): continue
            if typecode in 'FDG': continue
            for count in range(self.COUNT_MIN, 10):
                for rank in range(size):
                    ones = array([1]*count, typecode)
                    sbuf = array(range(count), typecode)
                    rbuf = array(-1, typecode, count+1)
                    for op in (MPI.SUM, MPI.PROD,
                               MPI.MAX, MPI.MIN,
                               MPI.REPLACE):
                        self.WIN.Lock(rank)
                        self.WIN.Put(ones.as_mpi(), rank)
                        self.WIN.Flush(rank)
                        r = self.WIN.Raccumulate(sbuf.as_mpi(),
                                                 rank, op=op)
                        r.Wait()
                        self.WIN.Flush(rank)
                        r = self.WIN.Rget(rbuf.as_mpi_c(count), rank)
                        r.Wait()
                        self.WIN.Unlock(rank)
                        #
                        for i in range(count):
                            self.assertEqual(sbuf[i], i)
                            self.assertEqual(rbuf[i], op(1, i))
                        self.assertEqual(rbuf[-1], -1)

    @unittest.skipMPI('openmpi(>=1.10,<1.11)')
    def testGetAccumulate(self):
        group = self.WIN.Get_group()
        size = group.Get_size()
        group.Free()
        for array, typecode in arrayimpl.subTest(self):
            if unittest.is_mpi_gpu('openmpi', array): continue
            if unittest.is_mpi_gpu('mvapich2', array): continue
            if typecode in 'FDG': continue
            for count in range(self.COUNT_MIN, 10):
                for rank in range(size):
                    ones = array([1]*count, typecode)
                    sbuf = array(range(count), typecode)
                    rbuf = array(-1, typecode, count+1)
                    gbuf = array(-1, typecode, count+1)
                    for op in (MPI.SUM, MPI.PROD,
                               MPI.MAX, MPI.MIN,
                               MPI.REPLACE, MPI.NO_OP):
                        self.WIN.Lock(rank)
                        self.WIN.Put(ones.as_mpi(), rank)
                        self.WIN.Flush(rank)
                        r = self.WIN.Rget_accumulate(sbuf.as_mpi(),
                                                     rbuf.as_mpi_c(count),
                                                     rank, op=op)
                        r.Wait()
                        self.WIN.Flush(rank)
                        r = self.WIN.Rget(gbuf.as_mpi_c(count), rank)
                        r.Wait()
                        self.WIN.Unlock(rank)
                        #
                        for i in range(count):
                            self.assertEqual(sbuf[i], i)
                            self.assertEqual(rbuf[i], 1)
                            self.assertEqual(gbuf[i], op(1, i))
                        self.assertEqual(rbuf[-1], -1)
                        self.assertEqual(gbuf[-1], -1)

    def testPutProcNull(self):
        rank = self.COMM.Get_rank()
        self.WIN.Lock(rank)
        r = self.WIN.Rput(None, MPI.PROC_NULL, None)
        r.Wait()
        self.WIN.Unlock(rank)

    def testGetProcNull(self):
        rank = self.COMM.Get_rank()
        self.WIN.Lock(rank)
        r = self.WIN.Rget(None, MPI.PROC_NULL, None)
        r.Wait()
        self.WIN.Unlock(rank)

    def testAccumulateProcNullReplace(self):
        rank = self.COMM.Get_rank()
        zeros = mkzeros(8)
        self.WIN.Lock(rank)
        r = self.WIN.Raccumulate([zeros, MPI.INT], MPI.PROC_NULL, None, MPI.REPLACE)
        r.Wait()
        r = self.WIN.Raccumulate([zeros, MPI.INT], MPI.PROC_NULL, None, MPI.REPLACE)
        r.Wait()
        self.WIN.Unlock(rank)

    def testAccumulateProcNullSum(self):
        rank = self.COMM.Get_rank()
        zeros = mkzeros(8)
        self.WIN.Lock(rank)
        r = self.WIN.Raccumulate([zeros, MPI.INT], MPI.PROC_NULL, None, MPI.SUM)
        r.Wait()
        r = self.WIN.Raccumulate([None, MPI.INT], MPI.PROC_NULL, None, MPI.SUM)
        r.Wait()
        self.WIN.Unlock(rank)


@unittest.skipMPI('MPI(<3.0)')
@unittest.skipMPI('openmpi(<1.8.1)')
@unittest.skipMPI('MPICH2(<1.5.0)')
class TestRMASelf(BaseTestRMA, unittest.TestCase):
    COMM = MPI.COMM_SELF

@unittest.skipMPI('MPI(<3.0)')
@unittest.skipMPI('openmpi(<1.8.1)')
@unittest.skipMPI('MPICH2(<1.5.0)')
class TestRMAWorld(BaseTestRMA, unittest.TestCase):
    COMM = MPI.COMM_WORLD


SpectrumMPI = MPI.get_vendor()[0] == 'Spectrum MPI'
try:
    if SpectrumMPI: raise NotImplementedError
    MPI.Win.Create(None, 1, MPI.INFO_NULL, MPI.COMM_SELF).Free()
except (NotImplementedError, MPI.Exception):
    unittest.disable(BaseTestRMA, 'mpi-rma-nb')


if __name__ == '__main__':
    unittest.main()



#------------------------------------------------------------------------------

cdef extern from "Python.h":
    int is_list   "PyList_Check" (object)
    int is_tuple  "PyTuple_Check" (object)

cdef extern from "Python.h":
    int PyIndex_Check(object)
    int PySequence_Check(object)
    object PyNumber_Index(object)
    Py_ssize_t PySequence_Size(object) except -1

cdef inline int is_integral(object ob):
    if not PyIndex_Check(ob): return 0
    if not PySequence_Check(ob): return 1
    try: PySequence_Size(ob)
    except: pass
    else: return 0
    try: PyNumber_Index(ob)
    except: return 0
    else: return 1

cdef inline int is_buffer(object ob):
    if PY3:
        return PyObject_CheckBuffer(ob)
    else:
        return PyObject_CheckBuffer(ob) or _Py2_IsBuffer(ob)

cdef inline int is_dlpack_buffer(object ob):
    return Py_CheckDLPackBuffer(ob)

cdef inline int is_cai_buffer(object ob):
    return Py_CheckCAIBuffer(ob)

cdef inline int is_datatype(object ob):
    if isinstance(ob, Datatype): return 1
    if PY3:
        if isinstance(ob, unicode): return 1
    else:
        if isinstance(ob, bytes): return 1
    return 0

#------------------------------------------------------------------------------

cdef extern from *:
    int INT_MAX

cdef inline int downcast(MPI_Aint value) except? -1:
    if value > <MPI_Aint>INT_MAX:
        raise OverflowError("integer %d does not fit in 'int'" % value)
    else:
        return <int>value

cdef inline int clipcount(MPI_Aint value):
    if value > <MPI_Aint>INT_MAX:
        return INT_MAX
    else:
        return <int>value

#------------------------------------------------------------------------------

@cython.final
@cython.internal
cdef class Bottom(int):
    """
    Type of `BOTTOM`
    """

    def __cinit__(self):
        cdef MPI_Aint a = self, b = <MPI_Aint>MPI_BOTTOM
        if a != b : raise ValueError("cannot create instance")

    def __repr__(self):
        return 'BOTTOM'


@cython.final
@cython.internal
cdef class InPlace(int):
    """
    Type of `IN_PLACE`
    """

    def __cinit__(self):
        cdef MPI_Aint a = self, b = <MPI_Aint>MPI_IN_PLACE
        if a != b : raise ValueError("cannot create instance")

    def __repr__(self):
        return 'IN_PLACE'


cdef object __BOTTOM__   = Bottom(<MPI_Aint>MPI_BOTTOM)

cdef object __IN_PLACE__ = InPlace(<MPI_Aint>MPI_IN_PLACE)


cdef inline bint is_BOTTOM(object obj):
    if PYPY: return type(obj) is type(__BOTTOM__) and obj == __BOTTOM__
    else:    return obj is __BOTTOM__

cdef inline bint is_IN_PLACE(object obj):
    if PYPY: return type(obj) is type(__IN_PLACE__) and obj == __IN_PLACE__
    else:    return obj is __IN_PLACE__

#------------------------------------------------------------------------------

@cython.final
@cython.internal
cdef class _p_message:
    cdef memory buf
    cdef object count
    cdef object displ
    cdef Datatype type

cdef _p_message message_basic(object o_buf,
                              object o_type,
                              bint readonly,
                              #
                              void        **baddr,
                              MPI_Aint     *bsize,
                              MPI_Datatype *btype,
                              ):
    cdef _p_message m = _p_message.__new__(_p_message)
    # special-case for BOTTOM or None,
    # an explicit MPI datatype is required
    if is_BOTTOM(o_buf) or o_buf is None:
        if isinstance(o_type, Datatype):
            m.type = <Datatype>o_type
        else:
            m.type = TypeDict[o_type]
        m.buf = newbuffer()
        baddr[0] = MPI_BOTTOM
        bsize[0] = 0
        btype[0] = m.type.ob_mpi
        return m
    # get buffer base address and length
    cdef bint fmt = (o_type is None)
    m.buf = getbuffer(o_buf, readonly, fmt)
    baddr[0] = <void*>    m.buf.view.buf
    bsize[0] = <MPI_Aint> m.buf.view.len
    # lookup datatype if not provided or not a Datatype
    if isinstance(o_type, Datatype):
        m.type = <Datatype>o_type
    elif o_type is None:
        m.type = TypeDict[getformat(m.buf)]
    else:
        m.type = TypeDict[o_type]
    btype[0] = m.type.ob_mpi
    # and we are done ...
    return m

cdef _p_message message_simple(object msg,
                               int readonly,
                               int rank,
                               int blocks,
                               #
                               void         **_addr,
                               int          *_count,
                               MPI_Datatype *_type,
                               ):
    # special-case PROC_NULL target rank
    if rank == MPI_PROC_NULL:
        _addr[0]  = NULL
        _count[0] = 0
        _type[0]  = MPI_BYTE
        return None
    # unpack message list/tuple
    cdef Py_ssize_t nargs = 0
    cdef object o_buf   = None
    cdef object o_count = None
    cdef object o_displ = None
    cdef object o_type  = None
    if is_buffer(msg):
        o_buf = msg
    elif is_list(msg) or is_tuple(msg):
        nargs = len(msg)
        if nargs == 2:
            (o_buf, o_count) = msg
            if is_datatype(o_count):
                (o_count, o_type) = None, o_count
            elif is_tuple(o_count) or is_list(o_count):
                (o_count, o_displ) = o_count
        elif nargs == 3:
            (o_buf, o_count, o_type) = msg
            if is_tuple(o_count) or is_list(o_count):
                (o_count, o_displ) = o_count
        elif nargs == 4:
            (o_buf, o_count, o_displ, o_type) = msg
        else:
            raise ValueError("message: expecting 2 to 4 items")
    elif is_dlpack_buffer(msg):
        o_buf = msg
    elif is_cai_buffer(msg):
        o_buf = msg
    elif PYPY:
        o_buf = msg
    else:
        raise TypeError("message: expecting buffer or list/tuple")
    # buffer: address, length, and datatype
    cdef void *baddr = NULL
    cdef MPI_Aint bsize = 0
    cdef MPI_Datatype btype = MPI_DATATYPE_NULL
    cdef _p_message m = message_basic(o_buf, o_type, readonly,
                                      &baddr, &bsize, &btype)
    # buffer: count and displacement
    cdef int count = 0 # number of datatype entries
    cdef int displ = 0 # from base buffer, in datatype entries
    cdef MPI_Aint extent = 0, lb = 0 # datatype extent
    cdef MPI_Aint length = bsize     # in bytes
    cdef MPI_Aint offset = 0         # from base buffer, in bytes
    if o_displ is not None:
        displ = <int> o_displ
        if displ < 0: raise ValueError(
            "message: negative diplacement %d" % displ)
        if displ > 0:
            if btype == MPI_DATATYPE_NULL: raise ValueError(
                "message: cannot handle diplacement, datatype is null")
            CHKERR( MPI_Type_get_extent(btype, &lb, &extent) )
            if extent <= 0: raise ValueError(
                ("message: cannot handle diplacement, "
                 "datatype extent %d (lb:%d, ub:%d)"
                 ) % (extent, lb, lb+extent))
            if displ*extent > length: raise ValueError(
                ("message: displacement %d out of bounds, "
                 "number of datatype entries %d"
                 ) % (displ, length//extent))
            offset = displ*extent
            length -= offset
    if o_count is not None:
        count = <int> o_count
        if count < 0: raise ValueError(
            "message: negative count %d" % count)
        if count > 0 and o_buf is None: raise ValueError(
            "message: buffer is None but count is %d" % count)
    elif length > 0:
        if extent == 0:
            if btype == MPI_DATATYPE_NULL: raise ValueError(
                "message: cannot infer count, datatype is null")
            CHKERR( MPI_Type_get_extent(btype, &lb, &extent) )
            if extent <= 0: raise ValueError(
                ("message: cannot infer count, "
                 "datatype extent %d (lb:%d, ub:%d)"
                 ) % (extent, lb, lb+extent))
        if (length % extent) != 0: raise ValueError(
            ("message: cannot infer count, "
             "buffer length %d is not a multiple of "
             "datatype extent %d (lb:%d, ub:%d)"
             ) % (length, extent, lb, lb+extent))
        if blocks < 2:
            count = downcast(length // extent)
        else:
            if ((length // extent) % blocks) != 0: raise ValueError(
                ("message: cannot infer count, "
                 "number of entries %d is not a multiple of "
                 "required number of blocks %d"
                 ) %  (length//extent, blocks))
            count = downcast((length // extent) // blocks)
    # return collected message data
    m.count = o_count if o_count is not None else count
    m.displ = o_displ if o_displ is not None else displ
    _addr[0]  = <void*>(<char*>baddr + offset)
    _count[0] = count
    _type[0]  = btype
    return m

cdef _p_message message_vector(object msg,
                               int readonly,
                               int rank,
                               int blocks,
                               #
                               void        **_addr,
                               int         **_counts,
                               int         **_displs,
                               MPI_Datatype *_type,
                               ):
    # special-case PROC_NULL target rank
    if rank == MPI_PROC_NULL:
        _addr[0]   = NULL
        _counts[0] = NULL
        _displs[0] = NULL
        _type[0]   = MPI_BYTE
        return None
    # unpack message list/tuple
    cdef Py_ssize_t nargs = 0
    cdef object o_buf    = None
    cdef object o_counts = None
    cdef object o_displs = None
    cdef object o_type   = None
    if is_buffer(msg):
        o_buf = msg
    elif is_list(msg) or is_tuple(msg):
        nargs = len(msg)
        if nargs == 2:
            (o_buf, o_counts) = msg
            if is_datatype(o_counts):
                (o_counts, o_type) = None, o_counts
            elif is_tuple(o_counts):
                (o_counts, o_displs) = o_counts
        elif nargs == 3:
            (o_buf, o_counts, o_type) = msg
            if is_tuple(o_counts):
                (o_counts, o_displs) = o_counts
        elif nargs == 4:
            (o_buf, o_counts, o_displs, o_type) = msg
        else:
            raise ValueError("message: expecting 2 to 4 items")
    elif is_dlpack_buffer(msg):
        o_buf = msg
    elif is_cai_buffer(msg):
        o_buf = msg
    elif PYPY:
        o_buf = msg
    else:
        raise TypeError("message: expecting buffer or list/tuple")
    # buffer: address, length, and datatype
    cdef void *baddr = NULL
    cdef MPI_Aint bsize = 0
    cdef MPI_Datatype btype = MPI_DATATYPE_NULL
    cdef _p_message m = message_basic(o_buf, o_type, readonly,
                                      &baddr, &bsize, &btype)
    # counts and displacements
    cdef int *counts = NULL
    cdef int *displs = NULL
    cdef int i=0, val=0
    cdef MPI_Aint extent=0, lb=0
    cdef MPI_Aint asize=0, aval=0
    if o_counts is None:
        if bsize > 0:
            if btype == MPI_DATATYPE_NULL: raise ValueError(
                "message: cannot infer count, "
                "datatype is null")
            CHKERR( MPI_Type_get_extent(btype, &lb, &extent) )
            if extent <= 0: raise ValueError(
                ("message: cannot infer count, "
                 "datatype extent %d (lb:%d, ub:%d)"
                 ) % (extent, lb, lb+extent))
            if (bsize % extent) != 0: raise ValueError(
                ("message: cannot infer count, "
                 "buffer length %d is not a multiple of "
                 "datatype extent %d (lb:%d, ub:%d)"
                 ) % (bsize, extent, lb, lb+extent))
            asize = bsize // extent
        o_counts = newarray(blocks, &counts)
        for i from 0 <= i < blocks:
            aval = (asize // blocks) + (asize % blocks > i)
            counts[i] = downcast(aval)
    elif is_integral(o_counts):
        val = <int> o_counts
        o_counts = newarray(blocks, &counts)
        for i from 0 <= i < blocks:
            counts[i] = val
    else:
        o_counts = chkarray(o_counts, blocks, &counts)
    if o_displs is None: # contiguous
        val = 0
        o_displs = newarray(blocks, &displs)
        for i from 0 <= i < blocks:
            displs[i] = val
            val += counts[i]
    elif is_integral(o_displs): # strided
        val = <int> o_displs
        o_displs = newarray(blocks, &displs)
        for i from 0 <= i < blocks:
            displs[i] = val * i
    else: # general
        o_displs = chkarray(o_displs, blocks, &displs)
    # return collected message data
    m.count = o_counts
    m.displ = o_displs
    _addr[0]   = baddr
    _counts[0] = counts
    _displs[0] = displs
    _type[0]   = btype
    return m

cdef tuple message_vector_w(object msg,
                            int readonly,
                            int blocks,
                            #
                            void         **_addr,
                            int          **_counts,
                            integral_t   **_displs,
                            MPI_Datatype **_types,
                            ):
    cdef int i = 0
    cdef Py_ssize_t nargs = len(msg)
    cdef object o_buffer, o_counts, o_displs, o_types
    if nargs == 2:
        o_buffer, o_types = msg
        o_counts = o_displs = None
    elif nargs == 3:
        o_buffer, (o_counts, o_displs), o_types = msg
    elif nargs == 4:
        o_buffer,  o_counts, o_displs,  o_types = msg
    else:
        raise ValueError("message: expecting 2 to 4 items")
    if is_BOTTOM(o_buffer):
        if o_counts is None:
            raise ValueError("message: BOTTOM requires counts")
        if o_displs is None:
            raise ValueError("message: BOTTOM requires displs")
        _addr[0] = MPI_BOTTOM
    elif readonly:
        o_buffer = getbuffer_r(o_buffer, _addr, NULL)
    else:
        o_buffer = getbuffer_w(o_buffer, _addr, NULL)
    if o_counts is None and o_displs is None:
        o_counts = newarray(blocks, _counts)
        o_displs = newarray(blocks, _displs)
        for i from 0 <= i < blocks:
            _counts[0][i] = 1
            _displs[0][i] = 0
    else:
        o_counts = chkarray(o_counts, blocks, _counts)
        o_displs = chkarray(o_displs, blocks, _displs)
    o_types  = asarray_Datatype(o_types, blocks, _types)
    return (o_buffer, o_counts, o_displs, o_types)

#------------------------------------------------------------------------------

@cython.final
@cython.internal
cdef class _p_msg_p2p:

    # raw C-side arguments
    cdef void         *buf
    cdef int          count
    cdef MPI_Datatype dtype
    # python-side argument
    cdef object _msg

    def __cinit__(self):
        self.buf   = NULL
        self.count = 0
        self.dtype = MPI_DATATYPE_NULL

    cdef int for_send(self, object msg, int rank) except -1:
        self._msg = message_simple(msg, 1, # readonly
                                   rank, 0,
                                   &self.buf,
                                   &self.count,
                                   &self.dtype)
        return 0

    cdef int for_recv(self, object msg, int rank) except -1:
        self._msg = message_simple(msg, 0, # writable
                                   rank, 0,
                                   &self.buf,
                                   &self.count,
                                   &self.dtype)
        return 0

cdef inline _p_msg_p2p message_p2p_send(object sendbuf, int dest):
    cdef _p_msg_p2p msg = _p_msg_p2p.__new__(_p_msg_p2p)
    msg.for_send(sendbuf, dest)
    return msg

cdef inline _p_msg_p2p message_p2p_recv(object recvbuf, int source):
    cdef _p_msg_p2p msg = _p_msg_p2p.__new__(_p_msg_p2p)
    msg.for_recv(recvbuf, source)
    return msg

#------------------------------------------------------------------------------

@cython.final
@cython.internal
cdef class _p_msg_cco:

    # raw C-side arguments
    cdef void *sbuf, *rbuf
    cdef int scount, rcount
    cdef int *scounts, *rcounts
    cdef int *sdispls, *rdispls
    cdef MPI_Datatype stype, rtype
    # python-side arguments
    cdef object _smsg, _rmsg
    cdef object _rcnt

    def __cinit__(self):
        self.sbuf    = self.rbuf    = NULL
        self.scount  = self.rcount  = 0
        self.scounts = self.rcounts = NULL
        self.sdispls = self.rdispls = NULL
        self.stype   = self.rtype   = MPI_DATATYPE_NULL

    # Collective Communication Operations
    # -----------------------------------

    # sendbuf arguments
    cdef int for_cco_send(self, bint VECTOR,
                          object amsg,
                          int rank, int blocks) except -1:
        cdef bint readonly = 1
        if not VECTOR: # block variant
            self._smsg = message_simple(
                amsg, readonly, rank, blocks,
                &self.sbuf, &self.scount, &self.stype)
        else: # vector variant
            self._smsg = message_vector(
                amsg, readonly, rank, blocks,
                &self.sbuf, &self.scounts,
                &self.sdispls, &self.stype)
        return 0

    # recvbuf arguments
    cdef int for_cco_recv(self, bint VECTOR,
                          object amsg,
                          int rank, int blocks) except -1:
        cdef bint readonly = 0
        if not VECTOR: # block variant
            self._rmsg = message_simple(
                amsg, readonly, rank, blocks,
                &self.rbuf, &self.rcount, &self.rtype)
        else: # vector variant
            self._rmsg = message_vector(
                amsg, readonly, rank, blocks,
                &self.rbuf, &self.rcounts,
                &self.rdispls, &self.rtype)
        return 0

    # bcast
    cdef int for_bcast(self,
                       object msg, int root,
                       MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int inter=0, rank=0, sending=0
        CHKERR( MPI_Comm_test_inter(comm, &inter) )
        if not inter: # intra-communication
            CHKERR( MPI_Comm_rank(comm, &rank) )
            if root == rank:
                self.for_cco_send(0, msg, root, 0)
                sending = 1
            else:
                self.for_cco_recv(0, msg, root, 0)
                sending = 0
        else: # inter-communication
            if (root == MPI_ROOT or
                root == MPI_PROC_NULL):
                self.for_cco_send(0, msg, root, 0)
                sending = 1
            else:
                self.for_cco_recv(0, msg, root, 0)
                sending = 0
        if sending:
            self.rbuf   = self.sbuf
            self.rcount = self.scount
            self.rtype  = self.stype
        else:
            self.sbuf   = self.rbuf
            self.scount = self.rcount
            self.stype  = self.rtype
        return 0

    # gather/gatherv
    cdef int for_gather(self, int v,
                        object smsg, object rmsg,
                        int root, MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int inter=0, size=0, rank=0, null=MPI_PROC_NULL
        CHKERR( MPI_Comm_test_inter(comm, &inter) )
        if not inter: # intra-communication
            CHKERR( MPI_Comm_size(comm, &size) )
            CHKERR( MPI_Comm_rank(comm, &rank) )
            if root == rank:
                self.for_cco_recv(v, rmsg, root, size)
                if is_IN_PLACE(smsg):
                    self.sbuf   = MPI_IN_PLACE
                    self.scount = self.rcount
                    self.stype  = self.rtype
                else:
                    self.for_cco_send(0, smsg, 0, 0)
            else:
                self.for_cco_recv(v, rmsg, null, size)
                self.for_cco_send(0, smsg, root, 0)
        else: # inter-communication
            CHKERR( MPI_Comm_remote_size(comm, &size) )
            if (root == MPI_ROOT or
                root == MPI_PROC_NULL):
                self.for_cco_recv(v, rmsg, root, size)
                self.for_cco_send(0, smsg, null, 0)
            else:
                self.for_cco_recv(v, rmsg, null, size)
                self.for_cco_send(0, smsg, root, 0)
        return 0

    # scatter/scatterv
    cdef int for_scatter(self, int v,
                         object smsg, object rmsg,
                         int root, MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int inter=0, size=0, rank=0, null=MPI_PROC_NULL
        CHKERR( MPI_Comm_test_inter(comm, &inter) )
        if not inter: # intra-communication
            CHKERR( MPI_Comm_size(comm, &size) )
            CHKERR( MPI_Comm_rank(comm, &rank) )
            if root == rank:
                self.for_cco_send(v, smsg, root, size)
                if is_IN_PLACE(rmsg):
                    self.rbuf   = MPI_IN_PLACE
                    self.rcount = self.scount
                    self.rtype  = self.stype
                else:
                    self.for_cco_recv(0, rmsg, root, 0)
            else:
                self.for_cco_send(v, smsg, null, size)
                self.for_cco_recv(0, rmsg, root, 0)
        else: # inter-communication
            CHKERR( MPI_Comm_remote_size(comm, &size) )
            if (root == MPI_ROOT or
                root == MPI_PROC_NULL):
                self.for_cco_send(v, smsg, root, size)
                self.for_cco_recv(0, rmsg, null,  0)
            else:
                self.for_cco_send(v, smsg, null, size)
                self.for_cco_recv(0, rmsg, root, 0)
        return 0

    # allgather/allgatherv
    cdef int for_allgather(self, int v,
                           object smsg, object rmsg,
                           MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int inter=0, size=0
        CHKERR( MPI_Comm_test_inter(comm, &inter) )
        if not inter: # intra-communication
            CHKERR( MPI_Comm_size(comm, &size) )
        else: # inter-communication
            CHKERR( MPI_Comm_remote_size(comm, &size) )
        #
        self.for_cco_recv(v, rmsg, 0, size)
        if not inter and is_IN_PLACE(smsg):
            self.sbuf   = MPI_IN_PLACE
            self.scount = self.rcount
            self.stype  = self.rtype
        else:
            self.for_cco_send(0, smsg, 0, 0)
        return 0

    # alltoall/alltoallv
    cdef int for_alltoall(self, int v,
                          object smsg, object rmsg,
                          MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int inter=0, size=0
        CHKERR( MPI_Comm_test_inter(comm, &inter) )
        if not inter: # intra-communication
            CHKERR( MPI_Comm_size(comm, &size) )
        else: # inter-communication
            CHKERR( MPI_Comm_remote_size(comm, &size) )
        #
        self.for_cco_recv(v, rmsg, 0, size)
        if not inter and is_IN_PLACE(smsg):
            self.sbuf    = MPI_IN_PLACE
            self.scount  = self.rcount
            self.scounts = self.rcounts
            self.sdispls = self.rdispls
            self.stype   = self.rtype
        else:
            self.for_cco_send(v, smsg, 0, size)
        return 0

    # Neighbor Collectives
    # --------------------

    # neighbor allgather/allgatherv
    cdef int for_neighbor_allgather(self, int v,
                                    object smsg, object rmsg,
                                    MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int recvsize=0
        comm_neighbors_count(comm, &recvsize, NULL)
        self.for_cco_send(0, smsg, 0, 0)
        self.for_cco_recv(v, rmsg, 0, recvsize)
        return 0

    # neighbor alltoall/alltoallv
    cdef int for_neighbor_alltoall(self, int v,
                                   object smsg, object rmsg,
                                   MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int sendsize=0, recvsize=0
        comm_neighbors_count(comm, &recvsize, &sendsize)
        self.for_cco_send(v, smsg, 0, sendsize)
        self.for_cco_recv(v, rmsg, 0, recvsize)
        return 0

    # Collective Reductions Operations
    # --------------------------------

    # sendbuf
    cdef int for_cro_send(self, object amsg, int root) except -1:
        self._smsg = message_simple(amsg, 1, # readonly
                                    root, 0,
                                    &self.sbuf,
                                    &self.scount,
                                    &self.stype)
        return 0

    # recvbuf
    cdef int for_cro_recv(self, object amsg, int root) except -1:
        self._rmsg = message_simple(amsg, 0, # writable
                                    root, 0,
                                    &self.rbuf,
                                    &self.rcount,
                                    &self.rtype)
        return 0

    cdef int for_reduce(self,
                        object smsg, object rmsg,
                        int root, MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int inter=0, rank=0, null=MPI_PROC_NULL
        CHKERR( MPI_Comm_test_inter(comm, &inter) )
        if not inter: # intra-communication
            CHKERR( MPI_Comm_rank(comm, &rank) )
            if root == rank:
                self.for_cro_recv(rmsg, root)
                if is_IN_PLACE(smsg):
                    self.sbuf   = MPI_IN_PLACE
                    self.scount = self.rcount
                    self.stype  = self.rtype
                else:
                    self.for_cro_send(smsg, root)
            else:
                self.for_cro_recv(rmsg, null)
                self.for_cro_send(smsg, root)
                self.rcount = self.scount
                self.rtype  = self.stype
        else: # inter-communication
            if (root == MPI_ROOT or
                root == MPI_PROC_NULL):
                self.for_cro_recv(rmsg, root)
                self.scount = self.rcount
                self.stype  = self.rtype
            else:
                self.for_cro_send(smsg, root)
                self.rcount = self.scount
                self.rtype  = self.stype
        return 0

    cdef int for_allreduce(self,
                           object smsg, object rmsg,
                           MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int inter=0
        CHKERR( MPI_Comm_test_inter(comm, &inter) )
        # get send and recv buffers
        self.for_cro_recv(rmsg, 0)
        if not inter and is_IN_PLACE(smsg):
            self.sbuf   = MPI_IN_PLACE
            self.scount = self.rcount
            self.stype  = self.rtype
        else:
            self.for_cro_send(smsg, 0)
        # check counts and datatypes
        if self.sbuf != MPI_IN_PLACE:
            if self.stype != self.rtype:
                raise ValueError(
                    "mismatch in send and receive MPI datatypes")
            if self.scount != self.rcount:
                raise ValueError(
                    "mismatch in send count %d and receive count %d" %
                    (self.scount, self.rcount))
        return 0

    cdef int for_reduce_scatter_block(self,
                                      object smsg, object rmsg,
                                      MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int inter=0, size=0
        CHKERR( MPI_Comm_test_inter(comm, &inter) )
        CHKERR( MPI_Comm_size(comm, &size) )
        # get send and recv buffers
        if not inter and is_IN_PLACE(smsg):
            self.for_cco_recv(0, rmsg, 0, size)
            self.sbuf = MPI_IN_PLACE
        else:
            self.for_cco_recv(0, rmsg, 0, 0)
            self.for_cco_send(0, smsg, 0, size)
        # check counts and datatypes
        if self.sbuf != MPI_IN_PLACE:
            if self.stype != self.rtype:
                raise ValueError(
                    "mismatch in send and receive MPI datatypes")
            if self.scount != self.rcount:
                raise ValueError(
                    "mismatch in send count %d receive count %d" %
                    (self.scount, self.rcount*size))
        return 0

    cdef int for_reduce_scatter(self,
                                object smsg, object rmsg, object rcnt,
                                MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int inter=0, size=0, rank=MPI_PROC_NULL
        CHKERR( MPI_Comm_test_inter(comm, &inter) )
        CHKERR( MPI_Comm_size(comm, &size) )
        CHKERR( MPI_Comm_rank(comm, &rank) )
        # get send and recv buffers
        self.for_cro_recv(rmsg, 0)
        if not inter and is_IN_PLACE(smsg):
            self.sbuf = MPI_IN_PLACE
        else:
            self.for_cro_send(smsg, 0)
        # get receive counts
        if rcnt is None and not inter and self.sbuf != MPI_IN_PLACE:
            self._rcnt = newarray(size, &self.rcounts)
            CHKERR( MPI_Allgather(&self.rcount, 1, MPI_INT,
                                  self.rcounts, 1, MPI_INT, comm) )
        else:
            self._rcnt = chkarray(rcnt, size, &self.rcounts)
        # total sum or receive counts
        cdef int i=0, sumrcounts=0
        for i from 0 <= i < size:
            sumrcounts += self.rcounts[i]
        # check counts and datatypes
        if self.sbuf != MPI_IN_PLACE:
            if self.stype != self.rtype:
                raise ValueError(
                    "mismatch in send and receive MPI datatypes")
            if self.scount != sumrcounts:
                raise ValueError(
                    "mismatch in send count %d and sum(counts) %d" %
                    (self.scount, sumrcounts))
            if self.rcount != self.rcounts[rank]:
                raise ValueError(
                    "mismatch in receive count %d and counts[%d] %d" %
                    (self.rcount, rank, self.rcounts[rank]))
        else:
            if self.rcount != sumrcounts:
                raise ValueError(
                    "mismatch in receive count %d and sum(counts) %d" %
                    (self.rcount, sumrcounts))
        return 0

    cdef int for_scan(self,
                      object smsg, object rmsg,
                      MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        # get send and recv buffers
        self.for_cro_recv(rmsg, 0)
        if is_IN_PLACE(smsg):
            self.sbuf   = MPI_IN_PLACE
            self.scount = self.rcount
            self.stype  = self.rtype
        else:
            self.for_cro_send(smsg, 0)
        # check counts and datatypes
        if self.sbuf != MPI_IN_PLACE:
            if self.stype != self.rtype:
                raise ValueError(
                    "mismatch in send and receive MPI datatypes")
            if self.scount != self.rcount:
                raise ValueError(
                    "mismatch in send count %d and receive count %d" %
                    (self.scount, self.rcount))
        return 0

    cdef int for_exscan(self,
                        object smsg, object rmsg,
                        MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        # get send and recv buffers
        self.for_cro_recv(rmsg, 0)
        if is_IN_PLACE(smsg):
            self.sbuf   = MPI_IN_PLACE
            self.scount = self.rcount
            self.stype  = self.rtype
        else:
            self.for_cro_send(smsg, 0)
        # check counts and datatypes
        if self.sbuf != MPI_IN_PLACE:
            if self.stype != self.rtype:
                raise ValueError(
                    "mismatch in send and receive MPI datatypes")
            if self.scount != self.rcount:
                raise ValueError(
                    "mismatch in send count %d and receive count %d" %
                    (self.scount, self.rcount))
        return 0


cdef inline _p_msg_cco message_cco():
    cdef _p_msg_cco msg = _p_msg_cco.__new__(_p_msg_cco)
    return msg

#------------------------------------------------------------------------------

@cython.final
@cython.internal
cdef class _p_msg_ccow:

    # raw C-side arguments
    cdef void *sbuf, *rbuf
    cdef int *scounts, *rcounts
    cdef int *sdispls, *rdispls
    cdef MPI_Aint *sdisplsA, *rdisplsA
    cdef MPI_Datatype *stypes, *rtypes
    # python-side arguments
    cdef object _smsg, _rmsg

    def __cinit__(self):
        self.sbuf     = self.rbuf     = NULL
        self.scounts  = self.rcounts  = NULL
        self.sdispls  = self.rdispls  = NULL
        self.sdisplsA = self.rdisplsA = NULL
        self.stypes   = self.rtypes   = NULL

    # alltoallw
    cdef int for_alltoallw(self,
                          object smsg, object rmsg,
                          MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int inter=0, size=0
        CHKERR( MPI_Comm_test_inter(comm, &inter) )
        if not inter: # intra-communication
            CHKERR( MPI_Comm_size(comm, &size) )
        else: # inter-communication
            CHKERR( MPI_Comm_remote_size(comm, &size) )
        #
        self._rmsg = message_vector_w(
            rmsg, 0, size,
            &self.rbuf, &self.rcounts,
            &self.rdispls, &self.rtypes)
        if not inter and is_IN_PLACE(smsg):
            self.sbuf    = MPI_IN_PLACE
            self.scounts = self.rcounts
            self.sdispls = self.rdispls
            self.stypes  = self.rtypes
            return 0
        self._smsg = message_vector_w(
            smsg, 1, size,
            &self.sbuf, &self.scounts,
            &self.sdispls, &self.stypes)
        return 0

    # neighbor alltoallw
    cdef int for_neighbor_alltoallw(self,
                                    object smsg, object rmsg,
                                    MPI_Comm comm) except -1:
        if comm == MPI_COMM_NULL: return 0
        cdef int sendsize=0, recvsize=0
        comm_neighbors_count(comm, &recvsize, &sendsize)
        self._rmsg = message_vector_w(
            rmsg, 0, recvsize,
            &self.rbuf, &self.rcounts,
            &self.rdisplsA, &self.rtypes)
        self._smsg = message_vector_w(
            smsg, 1, sendsize,
            &self.sbuf, &self.scounts,
            &self.sdisplsA, &self.stypes)
        return 0


cdef inline _p_msg_ccow message_ccow():
    cdef _p_msg_ccow msg = _p_msg_ccow.__new__(_p_msg_ccow)
    return msg

#------------------------------------------------------------------------------

@cython.final
@cython.internal
cdef class _p_msg_rma:

    # raw origin arguments
    cdef void*        oaddr
    cdef int          ocount
    cdef MPI_Datatype otype
    # raw compare arguments
    cdef void*        caddr
    cdef int          ccount
    cdef MPI_Datatype ctype
    # raw result arguments
    cdef void*        raddr
    cdef int          rcount
    cdef MPI_Datatype rtype
    # raw target arguments
    cdef MPI_Aint     tdisp
    cdef int          tcount
    cdef MPI_Datatype ttype
    # python-side arguments
    cdef object _origin
    cdef object _compare
    cdef object _result
    cdef object _target

    def __cinit__(self):
        self.oaddr  = NULL
        self.ocount = 0
        self.otype  = MPI_DATATYPE_NULL
        self.raddr  = NULL
        self.rcount = 0
        self.rtype  = MPI_DATATYPE_NULL
        self.tdisp  = 0
        self.tcount = 0
        self.ttype  = MPI_DATATYPE_NULL

    cdef int for_rma(self, int readonly,
                     object origin, int rank, object target) except -1:
        # ORIGIN
        self._origin = message_simple(
            origin, readonly, rank, 0,
            &self.oaddr,  &self.ocount,  &self.otype)
        if ((rank == MPI_PROC_NULL) and
            (origin is not None) and
            (is_list(origin) or is_tuple(origin)) and
            (len(origin) > 0 and isinstance(origin[-1], Datatype))):
            self.otype  = (<Datatype?>origin[-1]).ob_mpi
            self._origin = origin
        # TARGET
        cdef Py_ssize_t nargs = 0
        if target is None:
            self.tdisp  = 0
            self.tcount = self.ocount
            self.ttype  = self.otype
        elif is_integral(target):
            self.tdisp  = <MPI_Aint>target
            self.tcount = self.ocount
            self.ttype  = self.otype
        elif is_list(target) or is_tuple(target):
            self.tdisp  = 0
            self.tcount = self.ocount
            self.ttype  = self.otype
            nargs = len(target)
            if nargs >= 1:
                self.tdisp  = <MPI_Aint>target[0]
            if nargs >= 2:
                self.tcount = <int>target[1]
            if nargs >= 3:
                self.ttype  = (<Datatype?>target[2]).ob_mpi
            if nargs >= 4:
                raise ValueError("target: expecting 3 items at most")
        else:
            raise ValueError("target: expecting integral or list/tuple")
        self._target = target
        return 0

    cdef int for_put(self, object origin, int rank, object target) except -1:
        self.for_rma(1, origin, rank, target)
        return 0

    cdef int for_get(self, object origin, int rank, object target) except -1:
        self.for_rma(0, origin, rank, target)
        return 0

    cdef int for_acc(self, object origin, int rank, object target) except -1:
        self.for_rma(1, origin, rank, target)
        return 0

    cdef int set_origin(self, object origin, int rank) except -1:
        self._origin = message_simple(
            origin, 1, rank, 0,
            &self.oaddr,  &self.ocount,  &self.otype)
        self.tdisp  = 0
        self.tcount = self.ocount
        self.ttype  = self.otype

    cdef int set_compare(self, object compare, int rank) except -1:
        self._compare = message_simple(
            compare, 1, rank, 0,
            &self.caddr,  &self.ccount,  &self.ctype)

    cdef int set_result(self, object result, int rank) except -1:
        self._result = message_simple(
            result, 0, rank, 0,
            &self.raddr,  &self.rcount,  &self.rtype)

    cdef int for_get_acc(self, object origin, object result,
                         int rank, object target) except -1:
        self.for_rma(0, origin, rank, target)
        self.set_result(result, rank)
        return 0

    cdef int for_fetch_op(self, object origin, object result,
                          int rank, MPI_Aint disp) except -1:
        self.set_origin(origin, rank)
        self.set_result(result, rank)
        self.tdisp = disp
        if rank == MPI_PROC_NULL: return 0
        # Check
        if self.ocount != 1:  raise ValueError(
            "origin: expecting a single element, got %d" % self.ocount)
        if self.rcount != 1: raise ValueError(
            "result: expecting a single element, got %d" % self.rcount)
        if self.otype != self.rtype: raise ValueError(
            "mismatch in origin and result MPI datatypes")
        return 0

    cdef int for_cmp_swap(self, object origin, object compare, object result,
                          int rank, MPI_Aint disp) except -1:
        self.set_origin(origin, rank)
        self.set_compare(compare, rank)
        self.set_result(result, rank)
        self.tdisp = disp
        if rank == MPI_PROC_NULL: return 0
        # Check
        if self.ocount != 1:  raise ValueError(
            "origin: expecting a single element, got %d" % self.ocount)
        if self.ccount != 1:  raise ValueError(
            "compare: expecting a single element, got %d" % self.ccount)
        if self.rcount != 1: raise ValueError(
            "result: expecting a single element, got %d" % self.rcount)
        if self.otype != self.ctype: raise ValueError(
            "mismatch in origin and compare MPI datatypes")
        if self.otype != self.rtype: raise ValueError(
            "mismatch in origin and result MPI datatypes")
        return 0

cdef inline _p_msg_rma message_rma():
    cdef _p_msg_rma msg = _p_msg_rma.__new__(_p_msg_rma)
    return msg

#------------------------------------------------------------------------------

@cython.final
@cython.internal
cdef class _p_msg_io:

    # raw C-side data
    cdef void         *buf
    cdef int          count
    cdef MPI_Datatype dtype
    # python-side data
    cdef object _msg

    def __cinit__(self):
        self.buf   = NULL
        self.count = 0
        self.dtype = MPI_DATATYPE_NULL

    cdef int for_read(self, object msg) except -1:
        self._msg = message_simple(msg, 0, # writable
                                   0, 0,
                                   &self.buf,
                                   &self.count,
                                   &self.dtype)
        return 0

    cdef int for_write(self, object msg) except -1:
        self._msg = message_simple(msg, 1, # readonly
                                   0, 0,
                                   &self.buf,
                                   &self.count,
                                   &self.dtype)
        return 0

cdef inline _p_msg_io message_io_read(object buf):
    cdef _p_msg_io msg = _p_msg_io.__new__(_p_msg_io)
    msg.for_read(buf)
    return msg

cdef inline _p_msg_io message_io_write(object buf):
    cdef _p_msg_io msg = _p_msg_io.__new__(_p_msg_io)
    msg.for_write(buf)
    return msg

#------------------------------------------------------------------------------



#------------------------------------------------------------------------------

@cython.final
@cython.internal
cdef class _p_keyval:
    cdef public object copy_fn
    cdef public object delete_fn
    cdef public bint   nopython
    def __cinit__(self, copy_fn, delete_fn, nopython):
        if copy_fn   is False: copy_fn   = None
        if delete_fn is False: delete_fn = None
        if delete_fn is True:  delete_fn = None
        self.copy_fn   = copy_fn
        self.delete_fn = delete_fn
        self.nopython  = nopython

cdef dict type_keyval = {}
cdef dict comm_keyval = {}
cdef dict  win_keyval = {}

_keyval_registry = {
'Datatype' : type_keyval,
'Comm'     : comm_keyval,
'Win'      :  win_keyval,
}

#------------------------------------------------------------------------------

ctypedef fused PyMPI_attr_type:
    MPI_Datatype
    MPI_Comm
    MPI_Win

cdef inline object PyMPI_attr_call(
    object function,
    PyMPI_attr_type hdl,
    int keyval,
    object attrval):
    cdef object ob
    cdef object result
    if PyMPI_attr_type is MPI_Datatype:
        ob = new_Datatype(hdl)
    if PyMPI_attr_type is MPI_Comm:
        ob = new_Comm(hdl)
    if PyMPI_attr_type is MPI_Win:
        ob = new_Win (hdl)
    try:
        result = function(ob, keyval, attrval)
    finally:
        if PyMPI_attr_type is MPI_Datatype:
            (<Datatype>ob).ob_mpi = MPI_DATATYPE_NULL
        if PyMPI_attr_type is MPI_Comm:
            (<Comm>ob).ob_mpi = MPI_COMM_NULL
        if PyMPI_attr_type is MPI_Win:
            (<Win>ob).ob_mpi = MPI_WIN_NULL
    return result

cdef inline int PyMPI_attr_copy(
    PyMPI_attr_type hdl,
    int keyval,
    void *extra_state,
    void *attrval_in,
    void *attrval_out,
    int *flag) except -1:
    if flag != NULL: flag[0] = 0
    cdef _p_keyval state = <_p_keyval>extra_state
    if state.copy_fn is None: return 0
    cdef int p = not state.nopython
    if p: assert attrval_in != NULL
    cdef object attrval
    if p: attrval = <object>attrval_in
    else: attrval = PyLong_FromVoidPtr(attrval_in)
    if state.copy_fn is not True:
        attrval = PyMPI_attr_call(state.copy_fn, hdl, keyval, attrval)
        if attrval is NotImplemented: return 0
    cdef void **outval = <void **>attrval_out
    if p: outval[0] = <void *>attrval
    else: outval[0] = PyLong_AsVoidPtr(attrval)
    if flag != NULL: flag[0] = 1
    if p: Py_INCREF(attrval)
    Py_INCREF(state)
    return 0

cdef inline int PyMPI_attr_delete(
    PyMPI_attr_type hdl,
    int keyval,
    void *attrval_in,
    void *extra_state) except -1:
    cdef _p_keyval state = <_p_keyval>extra_state
    cdef int p = not state.nopython
    if p: assert attrval_in != NULL
    cdef object attrval
    if p: attrval = <object>attrval_in
    else: attrval = PyLong_FromVoidPtr(attrval_in)
    if state.delete_fn is not None:
        PyMPI_attr_call(state.delete_fn, hdl, keyval, attrval)
    if p: Py_DECREF(attrval)
    Py_DECREF(state)
    return 0

cdef inline int PyMPI_attr_copy_cb(
    PyMPI_attr_type hdl,
    int keyval,
    void *extra_state,
    void *attrval_in,
    void *attrval_out,
    int *flag,
    ) except MPI_ERR_UNKNOWN with gil:
    cdef int ierr = MPI_SUCCESS
    cdef object exc
    try:
        PyMPI_attr_copy(hdl, keyval, extra_state,
                        attrval_in, attrval_out, flag)
    except MPIException as exc:
        print_traceback()
        ierr = exc.Get_error_code()
    except:
        print_traceback()
        ierr = MPI_ERR_OTHER
    return ierr

cdef inline int PyMPI_attr_delete_cb(
    PyMPI_attr_type hdl,
    int keyval,
    void *attrval,
    void *extra_state,
    ) except MPI_ERR_UNKNOWN with gil:
    cdef int ierr = MPI_SUCCESS
    cdef object exc
    try:
        PyMPI_attr_delete(hdl, keyval, attrval, extra_state)
    except MPIException as exc:
        print_traceback()
        ierr = exc.Get_error_code()
    except:
        print_traceback()
        ierr = MPI_ERR_OTHER
    return ierr


@cython.callspec("MPIAPI")
cdef int PyMPI_attr_copy_fn(PyMPI_attr_type hdl,
                          int keyval,
                          void *extra_state,
                          void *attrval_in,
                          void *attrval_out,
                          int *flag) nogil:
    if flag != NULL: flag[0] = 0
    if extra_state == NULL:
        return MPI_ERR_INTERN
    if attrval_out == NULL:
        return MPI_ERR_INTERN
    if not Py_IsInitialized():
        return MPI_SUCCESS
    return PyMPI_attr_copy_cb(hdl, keyval, extra_state,
                              attrval_in, attrval_out, flag)

@cython.callspec("MPIAPI")
cdef int PyMPI_attr_delete_fn(PyMPI_attr_type hdl,
                            int keyval,
                            void *attrval,
                            void *extra_state) nogil:
    if extra_state == NULL:
        return MPI_ERR_INTERN
    if not Py_IsInitialized():
        return MPI_SUCCESS
    return PyMPI_attr_delete_cb(hdl, keyval, attrval, extra_state)

#------------------------------------------------------------------------------

cdef inline _p_keyval PyMPI_attr_state(
    PyMPI_attr_type hdl,
    int keyval):
    <void>hdl # unused
    if PyMPI_attr_type is MPI_Datatype:
        return <_p_keyval>type_keyval.get(keyval)
    elif PyMPI_attr_type is MPI_Comm:
        return <_p_keyval>comm_keyval.get(keyval)
    elif PyMPI_attr_type is MPI_Win:
        return <_p_keyval>win_keyval.get(keyval)

cdef inline object PyMPI_attr_get(
    PyMPI_attr_type hdl,
    int keyval,
    void *attrval):
    cdef _p_keyval state = PyMPI_attr_state(hdl, keyval)
    if state is not None and not state.nopython:
        return <object>attrval
    else:
        return PyLong_FromVoidPtr(attrval)

cdef inline int PyMPI_attr_set(
    PyMPI_attr_type hdl,
    int keyval,
    object attrval,
    ) except -1:
    cdef _p_keyval state = PyMPI_attr_state(hdl, keyval)
    cdef void *valptr = NULL
    if state is not None and not state.nopython:
        valptr = <void *>attrval
    else:
        valptr = PyLong_AsVoidPtr(attrval)
    if PyMPI_attr_type is MPI_Datatype:
        CHKERR( MPI_Type_set_attr(hdl, keyval, valptr) )
    if PyMPI_attr_type is MPI_Comm:
        CHKERR( MPI_Comm_set_attr(hdl, keyval, valptr) )
    if PyMPI_attr_type is MPI_Win:
        CHKERR( MPI_Win_set_attr(hdl, keyval, valptr) )
    if state is not None:
        if not state.nopython:
            Py_INCREF(attrval)
        Py_INCREF(state)
    return 0

#------------------------------------------------------------------------------



from libmpi import ffi, lib

NULL   = ffi.NULL
size_p = ffi.new('int*')
rank_p = ffi.new('int*')
nlen_p = ffi.new('int*')
name_p = ffi.new('char[]', lib.MPI_MAX_PROCESSOR_NAME);

lib.MPI_Init(NULL, NULL);

lib.MPI_Comm_size(lib.MPI_COMM_WORLD, size_p)
lib.MPI_Comm_rank(lib.MPI_COMM_WORLD, rank_p)
lib.MPI_Get_processor_name(name_p, nlen_p)

size = size_p[0]
rank = rank_p[0]
nlen = nlen_p[0]
name = ffi.string(name_p[0:nlen])

print("Hello, World! I am process %d of %d on %s."
      % (rank, size, name))

lib.MPI_Finalize()



from mpi4py import MPI
import mpiunittest as unittest


class TestStatus(unittest.TestCase):

    def setUp(self):
        self.STATUS = MPI.Status()

    def tearDown(self):
        self.STATUS = None

    def testDefaultFieldValues(self):
        self.assertEqual(self.STATUS.Get_source(), MPI.ANY_SOURCE)
        self.assertEqual(self.STATUS.Get_tag(),    MPI.ANY_TAG)
        self.assertEqual(self.STATUS.Get_error(),  MPI.SUCCESS)

    def testGetCount(self):
        count = self.STATUS.Get_count(MPI.BYTE)
        self.assertEqual(count, 0)

    def testGetElements(self):
        elements = self.STATUS.Get_elements(MPI.BYTE)
        self.assertEqual(elements, 0)

    def testSetElements(self):
        try:
            self.STATUS.Set_elements(MPI.BYTE, 7)
            count = self.STATUS.Get_count(MPI.BYTE)
            self.assertEqual(count, 7)
            elements = self.STATUS.Get_elements(MPI.BYTE)
            self.assertEqual(elements, 7)
        except NotImplementedError:
            if MPI.Get_version() >= (2,0): raise
            self.skipTest('mpi-status-set_elements')

    def testIsCancelled(self):
        flag = self.STATUS.Is_cancelled()
        self.assertTrue(type(flag) is bool)
        self.assertFalse(flag)

    def testSetCancelled(self):
        try:
            self.STATUS.Set_cancelled(True)
            flag = self.STATUS.Is_cancelled()
            self.assertTrue(flag)
        except NotImplementedError:
            if MPI.Get_version() >= (2,0): raise
            self.skipTest('mpi-status-set_cancelled')

    def testPyProps(self):
        self.assertEqual(self.STATUS.Get_source(), self.STATUS.source)
        self.assertEqual(self.STATUS.Get_tag(),    self.STATUS.tag)
        self.assertEqual(self.STATUS.Get_error(),  self.STATUS.error)
        self.STATUS.source = 1
        self.STATUS.tag    = 2
        self.STATUS.error  = MPI.ERR_ARG
        self.assertEqual(self.STATUS.source, 1)
        self.assertEqual(self.STATUS.tag,    2)
        self.assertEqual(self.STATUS.error,  MPI.ERR_ARG)

    def testConstructor(self):
        self.assertRaises(TypeError, MPI.Status, 123)
        self.assertRaises(TypeError, MPI.Status, "abc")
        
    def testCopyConstructor(self):
        self.STATUS.source = 1
        self.STATUS.tag    = 2
        self.STATUS.error  = MPI.ERR_ARG
        status = MPI.Status(self.STATUS)
        self.assertEqual(status.source, 1)
        self.assertEqual(status.tag,    2)
        self.assertEqual(status.error,  MPI.ERR_ARG)
        try:
            self.STATUS.Set_elements(MPI.BYTE, 7)
        except NotImplementedError:
            pass
        try:
            self.STATUS.Set_cancelled(True)
        except NotImplementedError:
            pass
        status = MPI.Status(self.STATUS)
        try:
            count = status.Get_count(MPI.BYTE)
            elems = status.Get_elements(MPI.BYTE)
            self.assertEqual(count, 7)
            self.assertEqual(elems, 7)
        except NotImplementedError:
            pass
        try:
            flag = status.Is_cancelled()
            self.assertTrue(flag)
        except NotImplementedError:
            pass


if __name__ == '__main__':
    unittest.main()



from mpi4py import MPI
import mpiunittest as unittest

from functools import reduce
cumsum  = lambda seq: reduce(lambda x, y: x+y, seq, 0)
cumprod = lambda seq: reduce(lambda x, y: x*y, seq, 1)

_basic = [None,
          True, False,
          -7, 0, 7, 2**31,
          -2**63+1, 2**63-1,
          -2.17, 0.0, 3.14,
          1+2j, 2-3j,
          'mpi4py',
          ]
messages = _basic
messages += [ list(_basic),
              tuple(_basic),
              dict([('k%d' % key, val)
                    for key, val in enumerate(_basic)])
              ]

class BaseTestCCOObj(object):

    COMM = MPI.COMM_NULL

    def testBarrier(self):
        self.COMM.barrier()

    def testBcast(self):
        for smess in messages:
            for root in range(self.COMM.Get_size()):
                rmess = self.COMM.bcast(smess, root=root)
                self.assertEqual(smess, rmess)

    def testGather(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for smess in messages + [messages]:
            for root in range(size):
                rmess = self.COMM.gather(smess, root=root)
                if rank == root:
                    self.assertEqual(rmess, [smess] * size)
                else:
                    self.assertEqual(rmess, None)

    def testScatter(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for smess in messages + [messages]:
            for root in range(size):
                if rank == root:
                    rmess = self.COMM.scatter([smess] * size, root=root)
                else:
                    rmess = self.COMM.scatter(None, root=root)
                self.assertEqual(rmess, smess)

    def testAllgather(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for smess in messages + [messages]:
            rmess = self.COMM.allgather(smess)
            self.assertEqual(rmess, [smess] * size)

    def testAlltoall(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for smess in messages + [messages]:
            rmess = self.COMM.alltoall([smess] * size)
            self.assertEqual(rmess, [smess] * size)

    def testReduce(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for root in range(size):
            for op in (MPI.SUM, MPI.PROD,
                       MPI.MAX, MPI.MIN,
                       MPI.MAXLOC, MPI.MINLOC,
                       MPI.REPLACE, MPI.NO_OP):
                if op == MPI.OP_NULL: continue
                if op in (MPI.MAXLOC, MPI.MINLOC):
                    sendobj = (rank, rank)
                else:
                    sendobj = rank
                value = self.COMM.reduce(sendobj, op=op, root=root)
                if rank != root:
                    self.assertTrue(value is None)
                else:
                    if op == MPI.SUM:
                        self.assertEqual(value, cumsum(range(size)))
                    elif op == MPI.PROD:
                        self.assertEqual(value, cumprod(range(size)))
                    elif op == MPI.MAX:
                        self.assertEqual(value, size-1)
                    elif op == MPI.MIN:
                        self.assertEqual(value, 0)
                    elif op == MPI.MAXLOC:
                        self.assertEqual(value[0], size-1)
                        self.assertEqual(value[1], size-1)
                    elif op == MPI.MINLOC:
                        self.assertEqual(value[0], 0)
                        self.assertEqual(value[1], 0)
                    elif op == MPI.REPLACE:
                        self.assertEqual(value, size-1)
                    elif op == MPI.NO_OP:
                        self.assertEqual(value, 0)

    def testAllreduce(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for op in (MPI.SUM, MPI.PROD,
                   MPI.MAX, MPI.MIN,
                   MPI.MAXLOC, MPI.MINLOC,
                   MPI.REPLACE, MPI.NO_OP):
            if op == MPI.OP_NULL: continue
            if op in (MPI.MAXLOC, MPI.MINLOC):
                sendobj = (rank, rank)
            else:
                sendobj = rank
            value = self.COMM.allreduce(sendobj, op)
            if op == MPI.SUM:
                self.assertEqual(value, cumsum(range(size)))
            elif op == MPI.PROD:
                self.assertEqual(value, cumprod(range(size)))
            elif op == MPI.MAX:
                self.assertEqual(value, size-1)
            elif op == MPI.MIN:
                self.assertEqual(value, 0)
            elif op == MPI.MAXLOC:
                self.assertEqual(value[1], size-1)
            elif op == MPI.MINLOC:
                self.assertEqual(value[1], 0)
            elif op == MPI.REPLACE:
                self.assertEqual(value, size-1)
            elif op == MPI.NO_OP:
                self.assertEqual(value, 0)

    def testScan(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        # --
        sscan = self.COMM.scan(size, op=MPI.SUM)
        self.assertEqual(sscan, cumsum([size]*(rank+1)))
        # --
        rscan = self.COMM.scan(rank, op=MPI.SUM)
        self.assertEqual(rscan, cumsum(range(rank+1)))
        # --
        minloc = self.COMM.scan((rank, rank), op=MPI.MINLOC)
        maxloc = self.COMM.scan((rank, rank), op=MPI.MAXLOC)
        self.assertEqual(minloc, (0, 0))
        self.assertEqual(maxloc, (rank, rank))
        # --
        if MPI.REPLACE != MPI.OP_NULL:
            rscan = self.COMM.scan(rank, op=MPI.REPLACE)
            self.assertEqual(rscan, rank)
        # --
        if MPI.NO_OP != MPI.OP_NULL:
            rscan = self.COMM.scan(rank, op=MPI.NO_OP)
            self.assertEqual(rscan, 0)

    def testExscan(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        # --
        sscan = self.COMM.exscan(size, op=MPI.SUM)
        if rank == 0:
            self.assertTrue(sscan is None)
        else:
            self.assertEqual(sscan, cumsum([size]*(rank)))
        # --
        rscan = self.COMM.exscan(rank, op=MPI.SUM)
        if rank == 0:
            self.assertTrue(rscan is None)
        else:
            self.assertEqual(rscan, cumsum(range(rank)))
        # --
        minloc = self.COMM.exscan((rank, rank), op=MPI.MINLOC)
        maxloc = self.COMM.exscan((rank, rank), op=MPI.MAXLOC)
        if rank == 0:
            self.assertEqual(minloc, None)
            self.assertEqual(maxloc, None)
        else:
            self.assertEqual(minloc, (0, 0))
            self.assertEqual(maxloc, (rank-1, rank-1))
        # --
        if MPI.REPLACE != MPI.OP_NULL:
            rscan = self.COMM.exscan(rank, op=MPI.REPLACE)
            if rank == 0:
                self.assertTrue(rscan is None)
            else:
                self.assertEqual(rscan, rank-1)
        # --
        if MPI.NO_OP != MPI.OP_NULL:
            rscan = self.COMM.exscan(rank, op=MPI.NO_OP)
            if rank == 0:
                self.assertTrue(rscan is None)
            else:
                self.assertEqual(rscan, 0)


class TestCCOObjSelf(BaseTestCCOObj, unittest.TestCase):
    COMM = MPI.COMM_SELF

class TestCCOObjWorld(BaseTestCCOObj, unittest.TestCase):
    COMM = MPI.COMM_WORLD

class TestCCOObjSelfDup(TestCCOObjSelf):
    def setUp(self):
        self.COMM = MPI.COMM_SELF.Dup()
    def tearDown(self):
        self.COMM.Free()

@unittest.skipMPI('openmpi(<1.4.0)', MPI.Query_thread() > MPI.THREAD_SINGLE)
class TestCCOObjWorldDup(TestCCOObjWorld):
    def setUp(self):
        self.COMM = MPI.COMM_WORLD.Dup()
    def tearDown(self):
        self.COMM.Free()


if __name__ == '__main__':
    unittest.main()



# http://mvapich.cse.ohio-state.edu/benchmarks/

from mpi4py import MPI

def osu_barrier(
    BENCHMARH = "MPI Barrier Latency Test",
    skip = 1000,
    loop = 10000,
    skip_large = 10,
    loop_large = 100,
    large_message_size = 8192,
    MAX_MSG_SIZE = 1<<20,
    ):

    comm = MPI.COMM_WORLD
    myid = comm.Get_rank()
    numprocs = comm.Get_size()

    if numprocs < 2:
        if myid == 0:
            errmsg = "This test requires at least two processes"
        else:
            errmsg = None
        raise SystemExit(errmsg)

    if myid == 0:
        print ('# %s' % (BENCHMARH,))
    if myid == 0:
        print ('# %-8s%20s' % ("Size [B]", "Latency [us]"))

    skip = skip_large
    loop = loop_large
    iterations = list(range(loop+skip))
    #
    comm.Barrier()
    for i in iterations:
        if i == skip:
            t_start = MPI.Wtime()
        comm.Barrier()
    t_end = MPI.Wtime()
    comm.Barrier()
    #
    if myid == 0:
        latency = (t_end - t_start) * 1e6 / loop
        print ('%-10d%20.2f' % (0, latency))


if __name__ == '__main__':
    osu_barrier()



from mpi4py import MPI
from array import array

master = MPI.Comm.Get_parent()
nprocs = master.Get_size()
myrank = master.Get_rank()

n  = array('i', [0])
master.Bcast([n, MPI.INT], root=0)
n = n[0]

h = 1.0 / n
s = 0.0
for i in range(myrank+1, n+1, nprocs):
    x = h * (i - 0.5)
    s += 4.0 / (1.0 + x**2)
pi = s * h

pi = array('d', [pi])
master.Reduce(sendbuf=[pi, MPI.DOUBLE],
              recvbuf=None,
              op=MPI.SUM, root=0)

master.Disconnect()



from mpi4py import MPI
import cffi
import os

_libdir = os.path.dirname(__file__)

ffi = cffi.FFI()
if MPI._sizeof(MPI.Comm) == ffi.sizeof('int'):
    _mpi_comm_t = 'int'
else:
    _mpi_comm_t = 'void*'
ffi.cdef("""
typedef %(_mpi_comm_t)s MPI_Comm;
void sayhello(MPI_Comm);
""" % vars())
lib = ffi.dlopen(os.path.join(_libdir, "libhelloworld.so"))

def sayhello(comm):
    comm_ptr = MPI._addressof(comm)
    comm_val = ffi.cast('MPI_Comm*', comm_ptr)[0]
    lib.sayhello(comm_val)



include "ExceptionP.pyx"
#include "ExceptionC.pyx"
MPIException = Exception


# Actually no errors
SUCCESS      = MPI_SUCCESS
ERR_LASTCODE = MPI_ERR_LASTCODE

# MPI-1 Error classes
# -------------------
# MPI-1 Objects
ERR_COMM      = MPI_ERR_COMM
ERR_GROUP     = MPI_ERR_GROUP
ERR_TYPE      = MPI_ERR_TYPE
ERR_REQUEST   = MPI_ERR_REQUEST
ERR_OP        = MPI_ERR_OP
# Communication argument parameters
ERR_BUFFER    = MPI_ERR_BUFFER
ERR_COUNT     = MPI_ERR_COUNT
ERR_TAG       = MPI_ERR_TAG
ERR_RANK      = MPI_ERR_RANK
ERR_ROOT      = MPI_ERR_ROOT
ERR_TRUNCATE  = MPI_ERR_TRUNCATE
# Multiple completion
ERR_IN_STATUS = MPI_ERR_IN_STATUS
ERR_PENDING   = MPI_ERR_PENDING
# Topology argument parameters
ERR_TOPOLOGY  = MPI_ERR_TOPOLOGY
ERR_DIMS      = MPI_ERR_DIMS
# Other arguments parameters
ERR_ARG       = MPI_ERR_ARG
# Other errors
ERR_OTHER     = MPI_ERR_OTHER
ERR_UNKNOWN   = MPI_ERR_UNKNOWN
ERR_INTERN    = MPI_ERR_INTERN

# MPI-2 Error classes
# -------------------
# MPI-2 Objects
ERR_INFO                   = MPI_ERR_INFO
ERR_FILE                   = MPI_ERR_FILE
ERR_WIN                    = MPI_ERR_WIN
# Object attributes
ERR_KEYVAL                 = MPI_ERR_KEYVAL
# Info Object
ERR_INFO_KEY               = MPI_ERR_INFO_KEY
ERR_INFO_VALUE             = MPI_ERR_INFO_VALUE
ERR_INFO_NOKEY             = MPI_ERR_INFO_NOKEY
# Input/Ouput
ERR_ACCESS                 = MPI_ERR_ACCESS
ERR_AMODE                  = MPI_ERR_AMODE
ERR_BAD_FILE               = MPI_ERR_BAD_FILE
ERR_FILE_EXISTS            = MPI_ERR_FILE_EXISTS
ERR_FILE_IN_USE            = MPI_ERR_FILE_IN_USE
ERR_NO_SPACE               = MPI_ERR_NO_SPACE
ERR_NO_SUCH_FILE           = MPI_ERR_NO_SUCH_FILE
ERR_IO                     = MPI_ERR_IO
ERR_READ_ONLY              = MPI_ERR_READ_ONLY
ERR_CONVERSION             = MPI_ERR_CONVERSION
ERR_DUP_DATAREP            = MPI_ERR_DUP_DATAREP
ERR_UNSUPPORTED_DATAREP    = MPI_ERR_UNSUPPORTED_DATAREP
ERR_UNSUPPORTED_OPERATION  = MPI_ERR_UNSUPPORTED_OPERATION
# Dynamic Process Management
ERR_NAME                   = MPI_ERR_NAME
ERR_NO_MEM                 = MPI_ERR_NO_MEM
ERR_NOT_SAME               = MPI_ERR_NOT_SAME
ERR_PORT                   = MPI_ERR_PORT
ERR_QUOTA                  = MPI_ERR_QUOTA
ERR_SERVICE                = MPI_ERR_SERVICE
ERR_SPAWN                  = MPI_ERR_SPAWN
# Windows
ERR_BASE         = MPI_ERR_BASE
ERR_SIZE         = MPI_ERR_SIZE
ERR_DISP         = MPI_ERR_DISP
ERR_ASSERT       = MPI_ERR_ASSERT
ERR_LOCKTYPE     = MPI_ERR_LOCKTYPE
ERR_RMA_CONFLICT = MPI_ERR_RMA_CONFLICT
ERR_RMA_SYNC     = MPI_ERR_RMA_SYNC
ERR_RMA_RANGE    = MPI_ERR_RMA_RANGE
ERR_RMA_ATTACH   = MPI_ERR_RMA_ATTACH
ERR_RMA_SHARED   = MPI_ERR_RMA_SHARED
ERR_RMA_FLAVOR   = MPI_ERR_RMA_FLAVOR



def Get_error_class(int errorcode: int) -> int:
    """
    Convert an *error code* into an *error class*
    """
    cdef int errorclass = MPI_SUCCESS
    CHKERR( MPI_Error_class(errorcode, &errorclass) )
    return errorclass

def Get_error_string(int errorcode: int) -> str:
    """
    Return the *error string* for a given
    *error class* or *error code*
    """
    cdef char string[MPI_MAX_ERROR_STRING+1]
    cdef int resultlen = 0
    CHKERR( MPI_Error_string(errorcode, string, &resultlen) )
    return tompistr(string, resultlen)


def Add_error_class() -> int:
    """
    Add an *error class* to the known error classes
    """
    cdef int errorclass = MPI_SUCCESS
    CHKERR( MPI_Add_error_class(&errorclass) )
    return errorclass

def Add_error_code(int errorclass: int) -> int:
    """
    Add an *error code* to an *error class*
    """
    cdef int errorcode = MPI_SUCCESS
    CHKERR( MPI_Add_error_code(errorclass, &errorcode) )
    return errorcode

def Add_error_string(int errorcode: int, string: str) -> None:
    """
    Associate an *error string* with an
    *error class* or *errorcode*
    """
    cdef char *cstring = NULL
    string = asmpistr(string, &cstring)
    CHKERR( MPI_Add_error_string(errorcode, cstring) )



from mpi4py import MPI
import mpiunittest as unittest
import arrayimpl


def maxvalue(a):
    try:
        typecode = a.typecode
    except AttributeError:
        typecode = a.dtype.char
    if typecode == ('f'):
        return 1e30
    elif typecode == ('d'):
        return 1e300
    else:
        return 2 ** (a.itemsize * 7) - 1


class BaseTestCCOVec(object):

    COMM = MPI.COMM_NULL

    def testGatherv(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size):
                    sbuf = array(root, typecode, count)
                    rbuf = array(  -1, typecode, size*size)
                    counts = [count] * size
                    displs = list(range(0, size*size, size))
                    recvbuf = rbuf.as_mpi_v(counts, displs)
                    if rank != root: recvbuf=None
                    self.COMM.Barrier()
                    self.COMM.Gatherv(sbuf.as_mpi(), recvbuf, root)
                    self.COMM.Barrier()
                    if recvbuf is not None:
                        for i in range(size):
                            row = rbuf[i*size:(i+1)*size]
                            a, b = row[:count], row[count:]
                            for va in a:
                                self.assertEqual(va, root)
                            for vb in b:
                                self.assertEqual(vb, -1)

    def testGatherv2(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size):
                    sbuf = array(root, typecode, size)
                    rbuf = array(  -1, typecode, size*size)
                    sendbuf = sbuf.as_mpi_c(count)
                    recvbuf = rbuf.as_mpi_v(count, size)
                    if rank != root: recvbuf=None
                    self.COMM.Barrier()
                    self.COMM.Gatherv(sendbuf, recvbuf, root)
                    self.COMM.Barrier()
                    if recvbuf is not None:
                        for i in range(size):
                            row = rbuf[i*size:(i+1)*size]
                            a, b = row[:count], row[count:]
                            for va in a:
                                self.assertEqual(va, root)
                            for vb in b:
                                self.assertEqual(vb, -1)

    def testGatherv3(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size+1):
                    #
                    sbuf = array(root, typecode, count).as_raw()
                    rbuf = array(  -1, typecode, count*size).as_raw()
                    sendbuf = sbuf
                    recvbuf = [rbuf, count]
                    if rank != root: recvbuf=None
                    self.COMM.Barrier()
                    self.COMM.Gatherv(sendbuf, recvbuf, root)
                    self.COMM.Barrier()
                    if recvbuf is not None:
                        for v in rbuf:
                            self.assertEqual(v, root)
                    #
                    sbuf = array(root, typecode, count).as_raw()
                    if rank == root:
                        rbuf = array(  -1, typecode, count*size).as_raw()
                    else:
                        rbuf = None
                    self.COMM.Gatherv(sbuf, rbuf, root)
                    self.COMM.Barrier()
                    if rank == root:
                        for v in rbuf:
                            self.assertEqual(v, root)

    def testScatterv(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size):
                    sbuf = array(root, typecode, size*size)
                    rbuf = array(  -1, typecode, count)
                    counts = [count] * size
                    displs = list(range(0, size*size, size))
                    sendbuf = sbuf.as_mpi_v(counts, displs)
                    if rank != root: sendbuf = None
                    self.COMM.Scatterv(sendbuf, rbuf.as_mpi(), root)
                    for vr in rbuf:
                        self.assertEqual(vr, root)

    def testScatterv2(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size):
                    sbuf = array(root, typecode, size*size)
                    rbuf = array(  -1, typecode, size)
                    sendbuf = sbuf.as_mpi_v(count, size)
                    recvbuf = rbuf.as_mpi_c(count)
                    if rank != root: sendbuf = None
                    self.COMM.Scatterv(sendbuf, recvbuf, root)
                    a, b = rbuf[:count], rbuf[count:]
                    for va in a:
                        self.assertEqual(va, root)
                    for vb in b:
                        self.assertEqual(vb, -1)

    def testScatterv3(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size+1):
                    #
                    sbuf = array(root, typecode, count*size).as_raw()
                    rbuf = array(  -1, typecode, count).as_raw()
                    sendbuf = [sbuf, count]
                    recvbuf = rbuf
                    if rank != root: sendbuf = None
                    self.COMM.Scatterv(sendbuf, recvbuf, root)
                    for v in rbuf:
                        self.assertEqual(v, root)
                    #
                    if rank == root:
                        sbuf = array(root, typecode, count*size).as_raw()
                    else:
                        sbuf = None
                    rbuf = array(  -1, typecode, count).as_raw()
                    self.COMM.Scatterv(sbuf, rbuf, root)
                    for v in rbuf:
                        self.assertEqual(v, root)

    def testAllgatherv(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size):
                    sbuf = array(root, typecode, count)
                    rbuf = array(  -1, typecode, size*size)
                    counts = [count] * size
                    displs = list(range(0, size*size, size))
                    sendbuf = sbuf.as_mpi()
                    recvbuf = rbuf.as_mpi_v(counts, displs)
                    self.COMM.Allgatherv(sendbuf, recvbuf)
                    for i in range(size):
                        row = rbuf[i*size:(i+1)*size]
                        a, b = row[:count], row[count:]
                        for va in a:
                            self.assertEqual(va, root)
                        for vb in b:
                            self.assertEqual(vb, -1)

    def testAllgatherv2(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size):
                    sbuf = array(root, typecode, size)
                    rbuf = array(  -1, typecode, size*size)
                    sendbuf = sbuf.as_mpi_c(count)
                    recvbuf = rbuf.as_mpi_v(count, size)
                    self.COMM.Allgatherv(sendbuf, recvbuf)
                    for i in range(size):
                        row = rbuf[i*size:(i+1)*size]
                        a, b = row[:count], row[count:]
                        for va in a:
                            self.assertEqual(va, root)
                        for vb in b:
                            self.assertEqual(vb, -1)

    def testAllgatherv3(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size+1):
                    #
                    sbuf = array(root, typecode, count).as_raw()
                    rbuf = array(  -1, typecode, count*size).as_raw()
                    sendbuf = sbuf
                    recvbuf = [rbuf, count]
                    self.COMM.Allgatherv(sendbuf, recvbuf)
                    for v in rbuf:
                        self.assertEqual(v, root)
                    #
                    sbuf = array(root, typecode, count).as_raw()
                    rbuf = array(  -1, typecode, count*size).as_raw()
                    self.COMM.Allgatherv(sbuf, rbuf)
                    for v in rbuf:
                        self.assertEqual(v, root)

    def testAlltoallv(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size):
                    sbuf = array(root, typecode, size*size)
                    rbuf = array(  -1, typecode, size*size)
                    counts = [count] * size
                    displs = list(range(0, size*size, size))
                    sendbuf = sbuf.as_mpi_v(counts, displs)
                    recvbuf = rbuf.as_mpi_v(counts, displs)
                    self.COMM.Alltoallv(sendbuf, recvbuf)
                    for i in range(size):
                        row = rbuf[i*size:(i+1)*size]
                        a, b = row[:count], row[count:]
                        for va in a:
                            self.assertEqual(va, root)
                        for vb in b:
                            self.assertEqual(vb, -1)

    def testAlltoallv2(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size):
                    sbuf = array(root, typecode, size*size)
                    rbuf = array(  -1, typecode, size*size)
                    sendbuf = sbuf.as_mpi_v(count, size)
                    recvbuf = rbuf.as_mpi_v(count, size)
                    self.COMM.Alltoallv(sendbuf, recvbuf)
                    for i in range(size):
                        row = rbuf[i*size:(i+1)*size]
                        a, b = row[:count], row[count:]
                        for va in a:
                            self.assertEqual(va, root)
                        for vb in b:
                            self.assertEqual(vb, -1)

    def testAlltoallv3(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for root in range(size):
                for count in range(size+1):
                    #
                    sbuf = array(root, typecode, count*size).as_raw()
                    rbuf = array(  -1, typecode, count*size).as_raw()
                    sendbuf = [sbuf, count]
                    recvbuf = [rbuf, count]
                    self.COMM.Alltoallv(sendbuf, recvbuf)
                    for v in rbuf:
                        self.assertEqual(v, root)
                    #
                    sbuf = array(root, typecode, count*size).as_raw()
                    rbuf = array(  -1, typecode, count*size).as_raw()
                    self.COMM.Alltoallv(sbuf, rbuf)
                    for v in rbuf:
                        self.assertEqual(v, root)

    def testAlltoallw(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for n in range(1,size+1):
                sbuf = array( n, typecode, (size, n))
                rbuf = array(-1, typecode, (size, n))
                sdt, rdt = sbuf.mpidtype, rbuf.mpidtype
                sdsp = list(range(0, size*n*sdt.extent, n*sdt.extent))
                rdsp = list(range(0, size*n*rdt.extent, n*rdt.extent))
                smsg = (sbuf.as_raw(), ([n]*size, sdsp), [sdt]*size)
                rmsg = (rbuf.as_raw(), ([n]*size, rdsp), [rdt]*size)
                try:
                    self.COMM.Alltoallw(smsg, rmsg)
                except NotImplementedError:
                    self.skipTest('mpi-alltoallw')
                for value in rbuf.flat:
                    self.assertEqual(value, n)

    def testAlltoallwBottom(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for n in range(1, size+1):
                sbuf = array( n, typecode, (size, n))
                rbuf = array(-1, typecode, (size, n))
                saddr = MPI.Get_address(sbuf.as_raw())
                raddr = MPI.Get_address(rbuf.as_raw())
                sdt, rdt = sbuf.mpidtype, rbuf.mpidtype
                stypes = [
                    MPI.Datatype.Create_struct([n], [saddr+d], [sdt]).Commit()
                    for d in list(range(0, size*n*sdt.extent, n*sdt.extent))
                ]
                rtypes = [
                    MPI.Datatype.Create_struct([n], [raddr+d], [sdt]).Commit()
                    for d in list(range(0, size*n*rdt.extent, n*rdt.extent))
                ]
                smsg = (MPI.BOTTOM, ([1]*size, [0]*size), stypes)
                rmsg = (MPI.BOTTOM, ([1]*size, [0]*size), rtypes)
                try:
                    self.COMM.Alltoallw(smsg, rmsg)
                except NotImplementedError:
                    self.skipTest('mpi-alltoallw')
                finally:
                    for t in stypes: t.Free()
                    for t in rtypes: t.Free()
                for value in rbuf.flat:
                    self.assertEqual(value, n)

@unittest.skipMPI('msmpi(<8.1.0)')
@unittest.skipMPI('openmpi(<1.8.0)')
@unittest.skipIf(MPI.BOTTOM == MPI.IN_PLACE, 'mpi-in-place')
class BaseTestCCOVecInplace(object):

    COMM = MPI.COMM_NULL

    def testAlltoallv(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for count in range(size):
                rbuf = array(-1, typecode, size*size)
                counts = [count] * size
                displs = list(range(0, size*size, size))
                for i in range(size):
                    for j in range(count):
                        rbuf[i*size+j] = rank
                recvbuf = rbuf.as_mpi_v(counts, displs)
                self.COMM.Alltoallv(MPI.IN_PLACE, recvbuf)
                for i in range(size):
                    row = rbuf[i*size:(i+1)*size]
                    a, b = row[:count], row[count:]
                    for va in a:
                        self.assertEqual(va, i)
                    for vb in b:
                        self.assertEqual(vb, -1)

    def testAlltoallw(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for count in range(size):
                rbuf = array(-1, typecode, size*size)
                for i in range(size):
                    for j in range(count):
                        rbuf[i*size+j] = rank
                rdt = rbuf.mpidtype
                rdsp = list(range(0, size*size*rdt.extent, size*rdt.extent))
                rmsg = (rbuf.as_raw(), ([count]*size, rdsp), [rdt]*size)
                try:
                    self.COMM.Alltoallw(MPI.IN_PLACE, rmsg)
                except NotImplementedError:
                    self.skipTest('mpi-alltoallw')
                for i in range(size):
                    row = rbuf[i*size:(i+1)*size]
                    a, b = row[:count], row[count:]
                    for va in a:
                        self.assertEqual(va, i)
                    for vb in b:
                        self.assertEqual(vb, -1)

    def testAlltoallw2(self):
        size = self.COMM.Get_size()
        rank = self.COMM.Get_rank()
        for array, typecode in arrayimpl.subTest(self):
            for count in range(size):
                rbuf = array(-1, typecode, size*size)
                for i in range(size):
                    for j in range(count):
                        rbuf[i*size+j] = rank
                rdt = rbuf.mpidtype
                rdsp = list(range(0, size*size*rdt.extent, size*rdt.extent))
                rmsg = (rbuf.as_raw(), [count]*size, rdsp, [rdt]*size)
                try:
                    self.COMM.Alltoallw(MPI.IN_PLACE, rmsg)
                except NotImplementedError:
                    self.skipTest('mpi-alltoallw')
                for i in range(size):
                    row = rbuf[i*size:(i+1)*size]
                    a, b = row[:count], row[count:]
                    for va in a:
                        self.assertEqual(va, i)
                    for vb in b:
                        self.assertEqual(vb, -1)


class TestCCOVecSelf(BaseTestCCOVec, unittest.TestCase):
    COMM = MPI.COMM_SELF

class TestCCOVecWorld(BaseTestCCOVec, unittest.TestCase):
    COMM = MPI.COMM_WORLD

class TestCCOVecSelfDup(TestCCOVecSelf):
    def setUp(self):
        self.COMM = MPI.COMM_SELF.Dup()
    def tearDown(self):
        self.COMM.Free()

class TestCCOVecWorldDup(TestCCOVecWorld):
    def setUp(self):
        self.COMM = MPI.COMM_WORLD.Dup()
    def tearDown(self):
        self.COMM.Free()

class TestCCOVecInplaceSelf(BaseTestCCOVecInplace, unittest.TestCase):
    COMM = MPI.COMM_SELF

class TestCCOVecInplaceWorld(BaseTestCCOVecInplace, unittest.TestCase):
    COMM = MPI.COMM_WORLD


if __name__ == '__main__':
    unittest.main()



from mpi4py import MPI
import mpiunittest as unittest


class TestInfoNull(unittest.TestCase):

    def testTruth(self):
        self.assertFalse(bool(MPI.INFO_NULL))

    def testPyMethods(self):
        inull = MPI.INFO_NULL
        def getitem(): return inull['k']
        def setitem(): inull['k'] = 'v'
        def delitem(): del inull['k']
        def update():  inull.update([])
        def pop():     inull.pop('k')
        def popitem(): inull.popitem()
        self.assertEqual(len(inull), 0)
        self.assertFalse('key' in inull)
        self.assertRaises(KeyError, getitem)
        self.assertRaises(KeyError, setitem)
        self.assertRaises(KeyError, delitem)
        self.assertRaises(KeyError, update)
        self.assertRaises(KeyError, pop)
        self.assertRaises(KeyError, popitem)
        self.assertEqual(inull.get('key', None), None)
        self.assertEqual(inull.pop('key', None), None)
        self.assertEqual(inull.keys(), [])
        self.assertEqual(inull.values(), [])
        self.assertEqual(inull.items(), [])
        self.assertEqual(inull.copy(), inull)
        inull.clear()

class TestInfoEnv(unittest.TestCase):

    def testTruth(self):
        self.assertTrue(bool(MPI.INFO_ENV))

    def testPyMethods(self):
        env = MPI.INFO_ENV
        if env == MPI.INFO_NULL: return
        for key in ("command", "argv",
                    "maxprocs", "soft",
                    "host", "arch",
                    "wdir", "file",
                    "thread_level"):
            v = env.Get(key)

class TestInfo(unittest.TestCase):

    def setUp(self):
        self.INFO  = MPI.Info.Create()

    def tearDown(self):
        self.INFO.Free()
        self.assertEqual(self.INFO, MPI.INFO_NULL)
        self.INFO = None

    def testTruth(self):
        self.assertTrue(bool(self.INFO))

    def testDup(self):
        info = self.INFO.Dup()
        self.assertNotEqual(self.INFO, info)
        self.assertEqual(info.Get_nkeys(), 0)
        info.Free()
        self.assertFalse(info)

    def testGet(self):
        value = self.INFO.Get('key')
        self.assertEqual(value, None)

    def testGetNKeys(self):
        self.assertEqual(self.INFO.Get_nkeys(), 0)

    def testGetSetDelete(self):
        INFO = self.INFO
        self.assertEqual(INFO.Get_nkeys(), 0)
        INFO.Set('key', 'value')
        nkeys = INFO.Get_nkeys()
        self.assertEqual(nkeys, 1)
        key = INFO.Get_nthkey(0)
        self.assertEqual(key, 'key')
        value = INFO.Get('key')
        self.assertEqual(value, 'value')
        INFO.Delete('key')
        nkeys = INFO.Get_nkeys()
        self.assertEqual(nkeys, 0)
        value = INFO.Get('key')
        self.assertEqual(value, None)

    def testPyMethods(self):
        INFO = self.INFO

        self.assertEqual(len(INFO), 0)
        self.assertTrue('key' not in INFO)
        self.assertEqual(INFO.keys(), [])
        self.assertEqual(INFO.values(), [])
        self.assertEqual(INFO.items(), [])

        INFO['key'] = 'value'
        self.assertEqual(len(INFO), 1)
        self.assertTrue('key' in INFO)
        self.assertEqual(INFO['key'], 'value')
        for key in INFO:
            self.assertEqual(key, 'key')
        self.assertEqual(INFO.keys(), ['key'])
        self.assertEqual(INFO.values(), ['value'])
        self.assertEqual(INFO.items(), [('key', 'value')])
        self.assertEqual(key, 'key')
        del INFO['key']
        self.assertEqual(len(INFO), 0)

        INFO['key'] = 'value'
        self.assertEqual(INFO.pop('key'), 'value')
        self.assertEqual(len(INFO), 0)
        self.assertEqual(INFO.pop('key', 'value'), 'value')
        self.assertRaises(KeyError, INFO.pop, 'key')
        INFO['key1'] = 'value1'
        INFO['key2'] = 'value2'
        self.assertEqual(INFO.pop('key1'), 'value1')
        self.assertEqual(len(INFO), 1)
        self.assertEqual(INFO.pop('key2'), 'value2')
        self.assertEqual(len(INFO), 0)

        INFO['key'] = 'value'
        self.assertEqual(INFO.popitem(), ('key', 'value'))
        self.assertEqual(len(INFO), 0)
        self.assertRaises(KeyError, INFO.popitem)
        INFO['key1'] = 'value1'
        INFO['key2'] = 'value2'
        self.assertEqual(INFO.popitem(), ('key2', 'value2'))
        self.assertEqual(len(INFO), 1)
        self.assertEqual(INFO.popitem(), ('key1', 'value1'))
        self.assertEqual(len(INFO), 0)

        self.assertEqual(len(INFO), 0)
        self.assertTrue('key' not in INFO)
        self.assertEqual(INFO.keys(), [])
        self.assertEqual(INFO.values(), [])
        self.assertEqual(INFO.items(), [])
        def getitem(): INFO['key']
        self.assertRaises(KeyError, getitem)
        def delitem(): del INFO['key']
        self.assertRaises(KeyError, delitem)

        INFO.clear()
        INFO.update([('key1','value1')])
        self.assertEqual(len(INFO), 1)
        self.assertEqual(INFO['key1'], 'value1')
        self.assertEqual(INFO.get('key1'), 'value1')
        self.assertEqual(INFO.get('key2'),  None)
        self.assertEqual(INFO.get('key2', 'value2'),  'value2')
        INFO.update(key2='value2')
        self.assertEqual(len(INFO), 2)
        self.assertEqual(INFO['key1'], 'value1')
        self.assertEqual(INFO['key2'], 'value2')
        self.assertEqual(INFO.get('key1'), 'value1')
        self.assertEqual(INFO.get('key2'), 'value2')
        self.assertEqual(INFO.get('key3'),  None)
        self.assertEqual(INFO.get('key3', 'value3'),  'value3')
        INFO.update([('key1', 'newval1')], key2='newval2')
        self.assertEqual(len(INFO), 2)
        self.assertEqual(INFO['key1'], 'newval1')
        self.assertEqual(INFO['key2'], 'newval2')
        self.assertEqual(INFO.get('key1'), 'newval1')
        self.assertEqual(INFO.get('key2'), 'newval2')
        self.assertEqual(INFO.get('key3'),  None)
        self.assertEqual(INFO.get('key3', 'newval3'),  'newval3')
        INFO.update(dict(key1='val1', key2='val2', key3='val3'))
        self.assertEqual(len(INFO), 3)
        self.assertEqual(INFO['key1'], 'val1')
        self.assertEqual(INFO['key2'], 'val2')
        self.assertEqual(INFO['key3'], 'val3')
        dupe = INFO.copy()
        self.assertEqual(INFO.items(), dupe.items())
        dupe.Free()
        INFO.clear()
        self.assertEqual(len(INFO), 0)
        self.assertEqual(INFO.get('key1'), None)
        self.assertEqual(INFO.get('key2'), None)
        self.assertEqual(INFO.get('key3'), None)
        self.assertEqual(INFO.get('key1', 'value1'), 'value1')
        self.assertEqual(INFO.get('key2', 'value2'), 'value2')
        self.assertEqual(INFO.get('key3', 'value3'), 'value3')


try:
    MPI.Info.Create().Free()
except NotImplementedError:
    unittest.disable(TestInfo, 'mpi-info')
    unittest.disable(TestInfoNull, 'mpi-info')
if (MPI.VERSION < 3 and MPI.INFO_ENV == MPI.INFO_NULL):
    unittest.disable(TestInfoEnv, 'mpi-info-env')


if __name__ == '__main__':
    unittest.main()
